#include "FrameInstruction.h"
#include "GameApp.h"

#include "../../../SexyAppFramework/WidgetManager.h"
#include "../../../SexyAppFramework/Color.h"
#include "../../../SexyAppFramework/DDImage.h"
#include "../../../SexyAppFramework/SysFont.h"
#include "../../../SexyAppFramework/ImageFont.h"
#include "../../../SexyAppFramework/ButtonWidget.h"

#include "../../../SexyAppFramework/SoundManager.h"
#include "../../../SexyAppFramework/BassMusicInterface.h"

using namespace Sexy;

FrameInstruction::FrameInstruction(GameApp* theApp)
{
	pApp = theApp;

	Resize(0, 0, pApp->mWidth, pApp->mHeight);

	mBtnNext = new ButtonWidget(BTN_Next, this);
	mBtnBack = new ButtonWidget(BTN_Back, this);
	mBtnSkip = new ButtonWidget(BTN_Skip, this);
	mBtnMainMenu = new ButtonWidget(BTN_MainMenu, this);

	mPageNum = 1;
}

FrameInstruction::~FrameInstruction()
{
	mBtnNext->mFont = NULL;
	delete mBtnNext;

	mBtnBack->mFont = NULL;
	delete mBtnBack;

	mBtnSkip->mFont = NULL;
	delete mBtnSkip;

	mBtnMainMenu->mFont = NULL;
	delete mBtnMainMenu;
}

void FrameInstruction::Draw(Graphics* g)
{
	g->SetColor(Color(159, 159, 127));
	g->FillRect(0, 0, mWidth, mHeight);

	g->SetColor(Color(159, 159, 127, 95));
	g->mColorizeImages=true;
	g->DrawImage(pApp->mGrayLogoImg, 0, 0, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 200, 0, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 400, 0, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 600, 0, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, -100, 200, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 100, 200, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 300, 200, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 500, 200, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 700, 200, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 0, 400, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 200, 400, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 400, 400, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 600, 400, 200, 200);
	g->mColorizeImages=false;
	
	//g->DrawImage(pApp->mMainLogoImg, 0, 0, 200,100);

	g->SetColor(Color(0, 0, 0));
	g->SetFont(pApp->mBigSysFont);

	switch(mPageNum)
	{
	case 1: 
			g->DrawImage(pApp->mTutorialImg1, 0,0);
			//WriteString(g, "The Pizza Parlor",	200, 40, 400, 0);
			break;

	case 2: 
			g->DrawImage(pApp->mTutorialImg2, 0,0);
			//WriteString(g, "How To Play",	200, 40, 400, 0);
			break;

	case 3: 
			g->DrawImage(pApp->mTutorialImg3, 0,0);
			//WriteString(g, "Advancing",	200, 40, 400, 0);
			break;
	}

	/*
	g->SetColor(Color(0, 0, 0));
	g->SetFont(pApp->mBigSysFont);
	WriteString(g, "by Jason Low",	500, 330, 300, -1);
	g->SetColor(Color(0, 0, 0));
	g->SetFont(pApp->mSysFont);
	WriteString(g, "blanko84@yahoo.com",	550, 350, 300, -1);

	g->SetColor(Color(255, 0, 0));
	g->SetFont(pApp->mSysFont);
	WriteString(g, pApp->mProductVersion,	0, 595, pApp->mWidth-5, 1);
	*/
}

void FrameInstruction::Update()
{
	Widget::Update();

	if(mUpdateCnt%100 == 0)
	{
		if(!pApp->mMusicInterface->IsPlaying(mPlayingSongNumber))
		{
			mPlayingSongNumber++;
			if(mPlayingSongNumber > 4)
			{
				mPlayingSongNumber = 0;
			}
			
			pApp->mMusicInterface->PlayMusic(mPlayingSongNumber);
		}
	}
}

void FrameInstruction::AddedToManager(WidgetManager* theWidgetManager)
{
	Widget::AddedToManager(theWidgetManager);

	mBtnBack->Resize(0, 500, 100, 100);
	mBtnBack->mFont = pApp->mBigSysFont;
	mBtnBack->mLabel = "Back";
	mBtnBack->SetColor(ButtonWidget::COLOR_LABEL, Color(255, 255, 255));
	mBtnBack->SetColor(ButtonWidget::COLOR_LABEL_HILITE, Color(255, 0, 0));
	mBtnBack->mButtonImage = pApp->mBigPizzaLogoImg1;
	mBtnBack->mDoFinger = true;
	theWidgetManager->AddWidget(mBtnBack);
	
	mBtnMainMenu->Resize(275, 500, 100, 100);
	mBtnMainMenu->mFont = pApp->mBigSysFont;
	mBtnMainMenu->mLabel = "Menu";
	mBtnMainMenu->SetColor(ButtonWidget::COLOR_LABEL, Color(255, 255, 255));
	mBtnMainMenu->SetColor(ButtonWidget::COLOR_LABEL_HILITE, Color(255, 0, 0));
	mBtnMainMenu->mButtonImage = pApp->mBigPizzaLogoImg2;
	mBtnMainMenu->mDoFinger = true;
	theWidgetManager->AddWidget(mBtnMainMenu);

	mBtnSkip->Resize(425, 500, 100, 100);
	mBtnSkip->mFont = pApp->mBigSysFont;
	mBtnSkip->mLabel = "Skip";
	mBtnSkip->SetColor(ButtonWidget::COLOR_LABEL, Color(255, 255, 255));
	mBtnSkip->SetColor(ButtonWidget::COLOR_LABEL_HILITE, Color(255, 0, 0));
	mBtnSkip->mButtonImage = pApp->mBigPizzaLogoImg2;
	mBtnSkip->mDoFinger = true;
	theWidgetManager->AddWidget(mBtnSkip);

	mBtnNext->Resize(700, 500, 100, 100);
	mBtnNext->mFont = pApp->mBigSysFont;
	mBtnNext->mLabel = "Next";
	mBtnNext->SetColor(ButtonWidget::COLOR_LABEL, Color(255, 255, 255));
	mBtnNext->SetColor(ButtonWidget::COLOR_LABEL_HILITE, Color(255, 0, 0));
	mBtnNext->mButtonImage = pApp->mBigPizzaLogoImg1;
	mBtnNext->mDoFinger = true;
	theWidgetManager->AddWidget(mBtnNext);

	mBtnBack->mVisible = false;


	mPlayingSongNumber = 2;
	pApp->mMusicInterface->PlayMusic(2);
	//pApp->mMusicInterface->FadeIn(2,0,-0.002, true);
	
}

void FrameInstruction::RemovedFromManager(WidgetManager* theWidgetManager)
{
	Widget::RemovedFromManager(theWidgetManager);
	
	theWidgetManager->RemoveWidget(mBtnBack);
	theWidgetManager->RemoveWidget(mBtnNext);
	theWidgetManager->RemoveWidget(mBtnSkip);
	theWidgetManager->RemoveWidget(mBtnMainMenu);

	pApp->mMusicInterface->FadeOut(mPlayingSongNumber);
}

void FrameInstruction::ButtonDepress(int theId)
{
	switch(theId)
	{
	case BTN_Next:  if(mPageNum < 3)
					{
						pApp->PlaySample(GameApp::SFX_PageTurn);
						mPageNum++;
						mBtnBack->mVisible = true;
						if(mPageNum == 3)
						{
							mBtnNext->mLabel = "Start";
						}
					}
					else if(mPageNum == 3)
					{
						pApp->PlaySample(GameApp::SFX_Click);
						pApp->ShowFrame(FrameFactory::GAME); 
					}
					break;

	case BTN_Back:	if(mPageNum > 1)
					{
						pApp->PlaySample(GameApp::SFX_PageTurn);
						mPageNum--;
						mBtnNext->mVisible = true;
						mBtnNext->mLabel = "Next";
						if(mPageNum == 1)
							mBtnBack->mVisible = false;
					}
					break; 

	case BTN_Skip:  
					pApp->PlaySample(GameApp::SFX_Click);
					pApp->ShowFrame(FrameFactory::GAME); 
					break;

	case BTN_MainMenu:  
						pApp->PlaySample(GameApp::SFX_Click);
						pApp->ShowFrame(FrameFactory::MAINMENU); 
						break;
	}
}
