#include "FrameMainMenu.h"
#include "GameApp.h"

#include "../../../SexyAppFramework/WidgetManager.h"
#include "../../../SexyAppFramework/Color.h"
#include "../../../SexyAppFramework/DDImage.h"
#include "../../../SexyAppFramework/SysFont.h"
#include "../../../SexyAppFramework/ImageFont.h"
#include "../../../SexyAppFramework/ButtonWidget.h"

#include "../../../SexyAppFramework/SoundManager.h"
#include "../../../SexyAppFramework/BassMusicInterface.h"

using namespace Sexy;

FrameMainMenu::FrameMainMenu(GameApp* theApp)
{
	pApp = theApp;

	Resize(0, 0, pApp->mWidth, pApp->mHeight);

	mBtnNewGame = new ButtonWidget(BTN_NewGame, this);
	mBtnQuitGame = new ButtonWidget(BTN_QuitGame, this);

	mPlayingSongNumber = 0;
}

FrameMainMenu::~FrameMainMenu()
{
	mBtnNewGame->mFont = NULL;
	delete mBtnNewGame;

	mBtnQuitGame->mFont = NULL;
	delete mBtnQuitGame;
}

void FrameMainMenu::Draw(Graphics* g)
{
	g->SetColor(Color(255, 190, 190));
	g->FillRect(0, 0, mWidth, mHeight);

	g->SetColor(Color(255, 190, 190, 95));
	g->mColorizeImages=true;
	g->DrawImage(pApp->mGrayLogoImg, 0, 0, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 200, 0, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 400, 0, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 600, 0, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, -100, 200, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 100, 200, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 300, 200, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 500, 200, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 700, 200, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 0, 400, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 200, 400, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 400, 400, 200, 200);
	g->DrawImage(pApp->mGrayLogoImg, 600, 400, 200, 200);
	g->mColorizeImages=false;
	
	g->DrawImage(pApp->mMainLogoImg, 100, 20);

	g->SetColor(Color(0, 0, 0));
	g->SetFont(pApp->mBigSysFont);
	WriteString(g, "by Jason Low",	500, 330, 300, -1);
	g->SetColor(Color(0, 0, 0));
	g->SetFont(pApp->mSysFont);
	WriteString(g, "blanko84@yahoo.com",	550, 350, 300, -1);


	g->SetColor(Color(255, 0, 0));
	g->SetFont(pApp->mSysFont);
	WriteString(g, "Developed using PopCap Games Framework 1.22 Beta",	0, 595, pApp->mWidth, -1);

	g->SetColor(Color(255, 0, 0));
	g->SetFont(pApp->mSysFont);
	WriteString(g, pApp->mProductVersion,	0, 595, pApp->mWidth-5, 1);
}

void FrameMainMenu::Update()
{
	Widget::Update();

	if(mUpdateCnt%100 == 0)
	{
		if(!pApp->mMusicInterface->IsPlaying(mPlayingSongNumber))
		{
			mPlayingSongNumber++;
			if(mPlayingSongNumber > 4)
			{
				mPlayingSongNumber = 0;
			}
			
			pApp->mMusicInterface->PlayMusic(mPlayingSongNumber);
		}
	}
}

void FrameMainMenu::AddedToManager(WidgetManager* theWidgetManager)
{
	Widget::AddedToManager(theWidgetManager);

	gSexyAppBase->SwitchScreenMode(true);

	mBtnNewGame->Resize(350, 350, 100, 100);
	mBtnNewGame->mFont = pApp->mBigSysFont;
	mBtnNewGame->mLabel = "Start";
	mBtnNewGame->SetColor(ButtonWidget::COLOR_LABEL, Color(255, 255, 255));
	mBtnNewGame->SetColor(ButtonWidget::COLOR_LABEL_HILITE, Color(255, 0, 0));
	mBtnNewGame->mButtonImage = pApp->mBigPizzaLogoImg1;
	//mBtnNewGame->mDownImage = pApp->pButtonDownImg;
	//mBtnNewGame->mOverImage = pApp->pButtonOverImg;
	mBtnNewGame->mDoFinger = true;
	theWidgetManager->AddWidget(mBtnNewGame);


	mBtnQuitGame->Resize(350, 460, 100, 100);
	mBtnQuitGame->mFont = pApp->mBigSysFont;
	mBtnQuitGame->mLabel = "Quit";
	mBtnQuitGame->SetColor(ButtonWidget::COLOR_LABEL, Color(255, 255, 255));
	mBtnQuitGame->SetColor(ButtonWidget::COLOR_LABEL_HILITE, Color(255, 0, 0));
	mBtnQuitGame->mButtonImage = pApp->mBigPizzaLogoImg2;
	//mBtnQuitGame->mDownImage = pApp->pButtonDownImg;
	//mBtnQuitGame->mOverImage = pApp->pButtonOverImg;
	mBtnQuitGame->mDoFinger = true;
	theWidgetManager->AddWidget(mBtnQuitGame);

	
	mPlayingSongNumber = 0;
	pApp->mMusicInterface->PlayMusic(0);
	//pApp->mMusicInterface->FadeIn(0,0,-0.002, true);

	
}

void FrameMainMenu::RemovedFromManager(WidgetManager* theWidgetManager)
{
	Widget::RemovedFromManager(theWidgetManager);
	
	theWidgetManager->RemoveWidget(mBtnQuitGame);
	theWidgetManager->RemoveWidget(mBtnNewGame);

	pApp->mMusicInterface->FadeOut(mPlayingSongNumber);
}

void FrameMainMenu::ButtonDepress(int theId)
{
	switch(theId)
	{
	case BTN_NewGame:	
						pApp->PlaySample(GameApp::SFX_Click);
						pApp->ShowFrame(FrameFactory::INSTRUCTION); 
						break;

	case BTN_QuitGame:	
						pApp->PlaySample(GameApp::SFX_Click);
						pApp->Shutdown();
						break;
	}
}
