#include "GameApp.h"
#include "GameFrame.h"
#include "FrameFactory.h"

#include "../../../SexyAppFramework/WidgetManager.h"
#include "../../../SexyAppFramework/SysFont.h"
#include "../../../SexyAppFramework/ImageFont.h"
#include "../../../SexyAppFramework/DDImage.h"

#include "../../../SexyAppFramework/SoundManager.h"
#include "../../../SexyAppFramework/BassMusicInterface.h"

using namespace Sexy;

GameApp::GameApp()
{
	mProdName = "Pizza Parlor";

	mProductVersion = "BETA 1.4";

	mTitle = mProdName + " - " + mProductVersion;

	mRegKey = "Pizza Parlor";

	mWidth = 800;
	mHeight = 600;

	mAutoEnable3D = true;
	mVSyncUpdates = true;

	mFrameFactory = new FrameFactory(this);
	pFrame = NULL;
	pGameFrame = NULL;

	mPizzaDoughImg = NULL;

	mBasilImg = NULL;
	mGarlicImg = NULL;
	mOreganoImg = NULL;
	mOnionImg = NULL;
	mAnchovyImg = NULL;
	mSausageImg = NULL;

	mPizzaMargheritaImg = NULL;
	mPizzaMarinaraImg = NULL;
	mPizzaCipollaImg = NULL;
	mPizzaNapoletanaImg = NULL;
	mPizzaVienneseImg = NULL;

	mOvenImg = NULL;
	mTableImg = NULL;
	mPrepTableImg = NULL;
	mFridgeImg = NULL;
	mTrashImg = NULL;

	mTileImg = NULL;

	for(int i=0; i<4; i++)
	{
		mChairImg[i] = NULL;

		mLorenzoIdleImg[i] = NULL;
		mLorenzoWalkImg[0][i] = NULL;
		mLorenzoWalkImg[1][i] = NULL;

		mCustomer1SitImg[i] = NULL;
		mCustomer2SitImg[i] = NULL;
		mCustomer3SitImg[i] = NULL;
		mCustomer4SitImg[i] = NULL;
		mCustomer5SitImg[i] = NULL;

		
		mPizzaTimerImg[i] = NULL;
	}

	mBigPizzaLogoImg1 = NULL;
	mBigPizzaLogoImg2 = NULL;

	mEuroImg = NULL;
	mHeartImg = NULL;

	mMainLogoImg = NULL;
	mGrayLogoImg = NULL;

	mMainMenuBtnDown = NULL;
		
	mMuteBtnDown = NULL;
	mUnmuteBtnDown = NULL;

	mResumeBtnDown = NULL;
	mPauseBtnDown = NULL;

	mFullscreenBtnDown = NULL;
	mWindowedBtnDown = NULL;

	mBigSysFont = NULL;
	mSysFont = NULL;
	mSmallSysFont = NULL;

	mTutorialImg1 = NULL;
	mTutorialImg2 = NULL;
	mTutorialImg3 = NULL;

}

GameApp::~GameApp()
{
	mWidgetManager->RemoveAllWidgets(false, true);

	delete mFrameFactory;
	
	delete mPizzaDoughImg;

	delete mBasilImg;
	delete mGarlicImg;
	delete mOreganoImg;
	delete mOnionImg;
	delete mAnchovyImg;
	delete mSausageImg;

	delete mPizzaMargheritaImg;
	delete mPizzaMarinaraImg;
	delete mPizzaCipollaImg;
	delete mPizzaNapoletanaImg;
	delete mPizzaVienneseImg;

	delete mOvenImg;
	delete mTableImg;
	delete mPrepTableImg;
	delete mFridgeImg;
	delete mTrashImg;

	delete mTileImg;

	for(int i=0; i<8; i++)
	{
		delete mLorenzoIdleImg[i];
	}

	for(int i=0; i<4; i++)
	{
		delete mChairImg[i];

		delete mLorenzoWalkImg[0][i];
		delete mLorenzoWalkImg[1][i];

		delete mCustomer1SitImg[i];
		delete mCustomer2SitImg[i];
		delete mCustomer3SitImg[i];
		delete mCustomer4SitImg[i];
		delete mCustomer5SitImg[i];

		delete mPizzaTimerImg[i];
	}

	delete mBigPizzaLogoImg1;
	delete mBigPizzaLogoImg2;

	delete mEuroImg;
	delete mHeartImg;

	delete mMainLogoImg;
	delete mGrayLogoImg;

	delete mMainMenuBtnDown;
	
	delete mMuteBtnDown;
	delete mUnmuteBtnDown;

	delete mResumeBtnDown;
	delete mPauseBtnDown;

	delete mFullscreenBtnDown;
	delete mWindowedBtnDown;

	delete mBigSysFont;
	delete mSysFont;
	delete mSmallSysFont;

	delete mTutorialImg1;
	delete mTutorialImg2;
	delete mTutorialImg3;

	mSoundManager->ReleaseSounds();
	mMusicInterface->UnloadAllMusic();
}

void GameApp::Init()
{
	SexyAppBase::Init();

	mSfxVolume = 0.50;
	mMusicVolume = 0.25;

	mMusicInterface->SetVolume(0.25);
	mSoundManager->SetVolume(0.50);

	gSexyAppBase->SwitchScreenMode(true);

	//SysFont* tempSysFont = new SysFont("Ariel", 12);
	//mBigSysFont = tempSysFont->CreateImageFont();

	mBigSysFont = new ImageFont(this, "fonts/MonotypeCorsiva20Bold.txt");
	if (!mBigSysFont->mFontData->mInitialized)	
	{
		delete mBigSysFont;
		mLoadingFailed = true;
		Popup("There was an error loading fonts/MonotypeCorsiva20Bold.txt");
		return;
	}
	mBigSysFont->Prepare();
	
	mSysFont = new ImageFont(this, "fonts/MonotypeCorsiva14Bold.txt");
	if (!mSysFont->mFontData->mInitialized)	
	{
		delete mSysFont;
		mLoadingFailed = true;
		Popup("There was an error loading fonts/MonotypeCorsiva14Bold.txt");
		return;
	}
	mSysFont->Prepare();

	mSmallSysFont = new ImageFont(this, "fonts/MonotypeCorsiva10Bold.txt");
	if (!mSmallSysFont->mFontData->mInitialized)	
	{
		delete mSmallSysFont;
		mLoadingFailed = true;
		Popup("There was an error loading fonts/MonotypeCorsiva10Bold.txt");
		return;
	}
	mSmallSysFont->Prepare();
}

void GameApp::LoadingThreadProc()
{
	mPizzaDoughImg = (DDImage*) GetImage("gfx/ingredients/pizzadough");
	if (mPizzaDoughImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/ingredients/pizzadough");
		return;
	}
	((DDImage*)mPizzaDoughImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();
	
	mBasilImg = (DDImage*) GetImage("gfx/ingredients/basil");
	if (mBasilImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/ingredients/basil");
		return;
	}
	((DDImage*)mBasilImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mOreganoImg = (DDImage*) GetImage("gfx/ingredients/oregano");
	if (mOreganoImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/ingredients/oregano");
		return;
	}
	((DDImage*)mOreganoImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mGarlicImg = (DDImage*) GetImage("gfx/ingredients/garlic");
	if (mGarlicImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/ingredients/garlic");
		return;
	}
	((DDImage*)mGarlicImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mOnionImg = (DDImage*) GetImage("gfx/ingredients/onions");
	if (mOnionImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/ingredients/onions");
		return;
	}
	((DDImage*)mOnionImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mAnchovyImg = (DDImage*) GetImage("gfx/ingredients/anchovies");
	if (mAnchovyImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/ingredients/anchovies");
		return;
	}
	((DDImage*)mAnchovyImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mSausageImg = (DDImage*) GetImage("gfx/ingredients/sausage");
	if (mSausageImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/ingredients/sausage");
		return;
	}
	((DDImage*)mSausageImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mPizzaMargheritaImg = (DDImage*) GetImage("gfx/pizza/pizza_Margherita");
	if (mPizzaMargheritaImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/pizza/pizza_Margherita");
		return;
	}
	((DDImage*)mPizzaMargheritaImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mPizzaMarinaraImg = (DDImage*) GetImage("gfx/pizza/pizza_Marinara");
	if (mPizzaMarinaraImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/pizza/pizza_Marinara");
		return;
	}
	((DDImage*)mPizzaMarinaraImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mPizzaCipollaImg = (DDImage*) GetImage("gfx/pizza/pizza_Cipolla");
	if (mPizzaCipollaImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/pizza/pizza_Cipolla");
		return;
	}
	((DDImage*)mPizzaCipollaImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mPizzaNapoletanaImg = (DDImage*) GetImage("gfx/pizza/pizza_Napoletana");
	if (mPizzaNapoletanaImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/pizza/pizza_Napoletana");
		return;
	}
	((DDImage*)mPizzaNapoletanaImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mPizzaVienneseImg = (DDImage*) GetImage("gfx/pizza/pizza_Viennese");
	if (mPizzaVienneseImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/pizza/pizza_Viennese");
		return;
	}
	((DDImage*)mPizzaVienneseImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();


	mOvenImg = (DDImage*) GetImage("gfx/objects/oven");
	if (mOvenImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/objects/oven");
		return;
	}
	((DDImage*)mOvenImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mTableImg = (DDImage*) GetImage("gfx/objects/table");
	if (mTableImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/objects/table");
		return;
	}
	((DDImage*)mTableImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mPrepTableImg = (DDImage*) GetImage("gfx/objects/preptable");
	if (mPrepTableImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/objects/preptable");
		return;
	}
	((DDImage*)mPrepTableImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mFridgeImg = (DDImage*) GetImage("gfx/objects/fridge");
	if (mFridgeImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/objects/fridge");
		return;
	}
	((DDImage*)mFridgeImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mTrashImg = (DDImage*) GetImage("gfx/objects/trashcan");
	if (mTrashImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/objects/trashcan");
		return;
	}
	((DDImage*)mTrashImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();


	mTileImg = (DDImage*) GetImage("gfx/tile/tile");
	if (mTileImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/tile/tile");
		return;
	}
	((DDImage*)mTileImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();


	for(int i=0; i<8; i++)
	{
		std::string path;
		path = "gfx/lorenzo/idle/lorenzo" + StrFormat("%d", i);
		mLorenzoIdleImg[i] = (DDImage*) GetImage(path);
		if (mLorenzoIdleImg[i] == NULL)
		{
			mLoadingFailed = true;
			Popup("There was an error loading the file: " + path);
			return;
		}
		((DDImage*)mLorenzoIdleImg[i])->Palletize();
		//mCompletedLoadingThreadTasks++;
		//pFrameLoading->MarkDirty();
	}

	for(int i=0; i<4; i++)
	{
		std::string path;
			
		path = "gfx/objects/chair/chair" + StrFormat("%d", i);
		mChairImg[i] = (DDImage*) GetImage(path);
		if (mChairImg[i] == NULL)
		{
			mLoadingFailed = true;
			Popup("There was an error loading the file: " + path);
			return;
		}
		((DDImage*)mChairImg[i])->Palletize();
		//mCompletedLoadingThreadTasks++;
		//pFrameLoading->MarkDirty();


		path = "gfx/lorenzo/walk0/lorenzo" + StrFormat("%d", i) + "_walk0";
		mLorenzoWalkImg[0][i] = (DDImage*) GetImage(path);
		if (mLorenzoWalkImg[0][i] == NULL)
		{
			mLoadingFailed = true;
			Popup("There was an error loading the file: " + path);
			return;
		}
		((DDImage*)mLorenzoWalkImg[0][i])->Palletize();
		//mCompletedLoadingThreadTasks++;
		//pFrameLoading->MarkDirty();

		path = "gfx/lorenzo/walk1/lorenzo" + StrFormat("%d", i) + "_walk1";
		mLorenzoWalkImg[1][i] = (DDImage*) GetImage(path);
		if (mLorenzoWalkImg[1][i] == NULL)
		{
			mLoadingFailed = true;
			Popup("There was an error loading the file: " + path);
			return;
		}
		((DDImage*)mLorenzoWalkImg[1][i])->Palletize();
		//mCompletedLoadingThreadTasks++;
		//pFrameLoading->MarkDirty();


		path = "gfx/customer/1/cust" + StrFormat("%d", i);
		mCustomer1SitImg[i] = (DDImage*) GetImage(path);
		if (mCustomer1SitImg[i] == NULL)
		{
			mLoadingFailed = true;
			Popup("There was an error loading the file: " + path);
			return;
		}
		((DDImage*)mCustomer1SitImg[i])->Palletize();
		//mCompletedLoadingThreadTasks++;
		//pFrameLoading->MarkDirty();

		path = "gfx/customer/2/cust" + StrFormat("%d", i);
		mCustomer2SitImg[i] = (DDImage*) GetImage(path);
		if (mCustomer2SitImg[i] == NULL)
		{
			mLoadingFailed = true;
			Popup("There was an error loading the file: " + path);
			return;
		}
		((DDImage*)mCustomer2SitImg[i])->Palletize();
		//mCompletedLoadingThreadTasks++;
		//pFrameLoading->MarkDirty();

		path = "gfx/customer/3/cust" + StrFormat("%d", i);
		mCustomer3SitImg[i] = (DDImage*) GetImage(path);
		if (mCustomer3SitImg[i] == NULL)
		{
			mLoadingFailed = true;
			Popup("There was an error loading the file: " + path);
			return;
		}
		((DDImage*)mCustomer3SitImg[i])->Palletize();
		//mCompletedLoadingThreadTasks++;
		//pFrameLoading->MarkDirty();

		path = "gfx/customer/4/cust" + StrFormat("%d", i);
		mCustomer4SitImg[i] = (DDImage*) GetImage(path);
		if (mCustomer4SitImg[i] == NULL)
		{
			mLoadingFailed = true;
			Popup("There was an error loading the file: " + path);
			return;
		}
		((DDImage*)mCustomer4SitImg[i])->Palletize();
		//mCompletedLoadingThreadTasks++;
		//pFrameLoading->MarkDirty();

		path = "gfx/customer/5/cust" + StrFormat("%d", i);
		mCustomer5SitImg[i] = (DDImage*) GetImage(path);
		if (mCustomer5SitImg[i] == NULL)
		{
			mLoadingFailed = true;
			Popup("There was an error loading the file: " + path);
			return;
		}
		((DDImage*)mCustomer5SitImg[i])->Palletize();
		//mCompletedLoadingThreadTasks++;
		//pFrameLoading->MarkDirty();


		path = "gfx/UI/PizzaTimer" + StrFormat("%d", i+1);
		mPizzaTimerImg[i] = (DDImage*) GetImage(path);
		if (mPizzaTimerImg[i] == NULL)
		{
			mLoadingFailed = true;
			Popup("There was an error loading the file: " + path);
			return;
		}
		((DDImage*)mPizzaTimerImg[i])->Palletize();
		//mCompletedLoadingThreadTasks++;
		//pFrameLoading->MarkDirty();
	}

	mBigPizzaLogoImg1 = (DDImage*) GetImage("gfx/UI/BigPizzaLogo1");
	if (mBigPizzaLogoImg1 == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/BigPizzaLogo1");
		return;
	}
	((DDImage*)mBigPizzaLogoImg1)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mBigPizzaLogoImg2 = (DDImage*) GetImage("gfx/UI/BigPizzaLogo2");
	if (mBigPizzaLogoImg2 == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/BigPizzaLogo2");
		return;
	}
	((DDImage*)mBigPizzaLogoImg2)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mEuroImg = (DDImage*) GetImage("gfx/UI/Euro");
	if (mEuroImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/Euro");
		return;
	}
	((DDImage*)mEuroImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mHeartImg = (DDImage*) GetImage("gfx/UI/Heart");
	if (mHeartImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/Heart");
		return;
	}
	((DDImage*)mHeartImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mMainLogoImg = (DDImage*) GetImage("gfx/UI/MainLogo");
	if (mMainLogoImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/MainLogo");
		return;
	}
	((DDImage*)mMainLogoImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mGrayLogoImg = (DDImage*) GetImage("gfx/UI/GrayLogo");
	if (mGrayLogoImg == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/GrayLogo");
		return;
	}
	((DDImage*)mGrayLogoImg)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mMainMenuBtnDown = (DDImage*) GetImage("gfx/UI/MainMenuBtnDown");
	if (mMainMenuBtnDown == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/MainMenuBtnDown");
		return;
	}
	((DDImage*)mMainMenuBtnDown)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mMuteBtnDown = (DDImage*) GetImage("gfx/UI/MuteBtnDown");
	if (mMuteBtnDown == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/MuteBtnDown");
		return;
	}
	((DDImage*)mMuteBtnDown)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mUnmuteBtnDown = (DDImage*) GetImage("gfx/UI/UnmuteBtnDown");
	if (mUnmuteBtnDown == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/UnmuteBtnDown");
		return;
	}
	((DDImage*)mUnmuteBtnDown)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mResumeBtnDown = (DDImage*) GetImage("gfx/UI/ResumeBtnDown");
	if (mResumeBtnDown == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/ResumeBtnDown");
		return;
	}
	((DDImage*)mResumeBtnDown)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mPauseBtnDown = (DDImage*) GetImage("gfx/UI/PauseBtnDown");
	if (mPauseBtnDown == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/PauseBtnDown");
		return;
	}
	((DDImage*)mPauseBtnDown)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mFullscreenBtnDown = (DDImage*) GetImage("gfx/UI/FullscreenBtnDown");
	if (mFullscreenBtnDown == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/FullscreenBtnDown");
		return;
	}
	((DDImage*)mFullscreenBtnDown)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mWindowedBtnDown = (DDImage*) GetImage("gfx/UI/WindowedBtnDown");
	if (mWindowedBtnDown == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/UI/WindowedBtnDown");
		return;
	}
	((DDImage*)mWindowedBtnDown)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();


	mTutorialImg1 = (DDImage*) GetImage("gfx/tutorial/tut1");
	if (mTutorialImg1 == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/tutorial/tut1");
		return;
	}
	((DDImage*)mTutorialImg1)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mTutorialImg2 = (DDImage*) GetImage("gfx/tutorial/tut2");
	if (mTutorialImg2 == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/tutorial/tut2");
		return;
	}
	((DDImage*)mTutorialImg2)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mTutorialImg3 = (DDImage*) GetImage("gfx/tutorial/tut3");
	if (mTutorialImg3 == NULL)
	{
		mLoadingFailed = true;
		Popup("There was an error loading the file: gfx/tutorial/tut3");
		return;
	}
	((DDImage*)mTutorialImg3)->Palletize();
	//mCompletedLoadingThreadTasks++;
	//pFrameLoading->MarkDirty();

	mMusicInterface->LoadMusic(0, "music/Andromeda_Polka.ogg");
	mMusicInterface->LoadMusic(1, "music/Anna_Polka.ogg");
	mMusicInterface->LoadMusic(2, "music/Peperocina_Polka.ogg");
	mMusicInterface->LoadMusic(3, "music/Desperata_Polka.ogg");
	mMusicInterface->LoadMusic(4, "music/ScaccoMatto_Polka.ogg");

	if (!mSoundManager->LoadSound(SFX_CashRegister, "sfx/cashregister"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/cashregister");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_Clap, "sfx/clap"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/clap");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_Click, "sfx/click"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/click");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_ClockTick, "sfx/clocktick"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/clocktick");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_CrowdOoh, "sfx/crowdooh"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/crowdooh");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_Ding, "sfx/ding"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/ding");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_DoorSlam, "sfx/doorslam"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/doorslam");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_HeartBeat, "sfx/heartbeat"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/heartbeat");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_HurryUp, "sfx/hurryup"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/hurryup");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_PageTurn, "sfx/pageturn"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/pageturn");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_Thud, "sfx/thud"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/thud");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_TrashCan, "sfx/trashcan"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/trashcan");
		return;
	}

	if (!mSoundManager->LoadSound(SFX_WhistleHere, "sfx/whistlehere"))
	{
		mLoadingFailed = true;		
		Popup("There was an error loading sfx/whistlehere");
		return;
	}


}

void GameApp::LoadingThreadCompleted()
{
	SexyAppBase::LoadingThreadCompleted();

	if (mLoadingFailed)
		return;

	ShowFrame(mFrameFactory->MAINMENU);
}

void GameApp::LostFocus()
{
	if (pGameFrame != NULL)
	{
		pGameFrame->TogglePauseGame(true);
	}
}

void GameApp::GotFocus()
{
	if (pGameFrame != NULL)
	{
		//pGameFrame->TogglePauseGame();
	}
}

void GameApp::ShowFrame(FrameFactory::eFrameName name)
{
	if(pFrame!=NULL)
	{
		//mWidgetManager->RemoveWidget(pFrame);
		mWidgetManager->RemoveAllWidgets();
		SafeDeleteWidget(pFrame);	
		pFrame = NULL;
		pGameFrame = NULL;
	}
	
	if(name == mFrameFactory->GAME)
	{
		pGameFrame = new GameFrame(this);
		pFrame = pGameFrame;
	}
	else
	{
		pFrame = mFrameFactory->getFrame(name);
	}

	if(pFrame == NULL)
	{
		Popup("There was an error in FrameFactory. Check frame code. (Code " + StrFormat("%d", name) + ")");
	}
	mWidgetManager->AddWidget(pFrame);
}