#include "GameFrame.h"
#include "GameApp.h"

#include "GameSpace.h"
#include "GameUI.h"
#include "GameOverlay.h"

#include "PizzaTimer.h"

#include "GameObjectFactory.h"
#include "TObjectFactory.h"
#include "GameCharFactory.h"

#include "TObject.h"
#include "GameObject.h"
#include "GameChar.h"

#include "GameNode.h"

#include "Customer.h"
#include "ObjectTable.h"

#include "Tray.h"
#include "PopupMsg.h"

#include "../../../SexyAppFramework/WidgetManager.h"

#include "../../../SexyAppFramework/SoundManager.h"
#include "../../../SexyAppFramework/BassMusicInterface.h"

using namespace Sexy;

GameFrame::GameFrame(GameApp* theApp)
{
	pApp = theApp;

	Resize(0, 0, pApp->mWidth, pApp->mHeight);

	mMouseX = mMouseY = 0;

	mGameObjectFactory = new GameObjectFactory(pApp, this);
	mTObjectFactory = new TObjectFactory(pApp, this);
	mGameCharFactory = new GameCharFactory(pApp, this);

	pSelectedChar = NULL;

	mColMapSizeX = 15;
	mColMapSizeY = 10;

	mColMap = new bool*[mColMapSizeX];
	for(short i=0; i<mColMapSizeX; i++) 
	{
		mColMap[i] = new bool[mColMapSizeY];
		for(short j=0; j<mColMapSizeY; j++) 
		{
			mColMap[i][j] = 0;
		}
	}

	mOffsetX = 25;
	mOffsetY = 25;

	mGridSize = 50;

	mCountdown = 100;

	mCustomerArraySize = 0;
	mCustomerArray = NULL;

	mTableArraySize = 0;
	mTableArray = NULL;

	mObjectOven1 = NULL;
	mObjectOven2 = NULL;
	mObjectOven3 = NULL;
	mObjectOven4 = NULL;

	mObjectPrepTable1 = NULL;
	mObjectPrepTable2 = NULL;
	mObjectPrepTable3 = NULL;
	mObjectPrepTable4 = NULL;

	mObjectFridge = NULL;
	mObjectTrash = NULL;

	mCharLorenzo = NULL;

	mTray = new Tray(pApp, this, 4);
	pCursorTrayWidget = mTray->getCursorWidget();

	mTimer = 0;
	mFunds = 100;
	mReputation = 100;

	mPaused = false;
	mGamePaused = false;

	mGoalLine = 120;
	mDangerLine = 50;
	mMaxReputation = 200;

	mDayDuration = 12000;
	mLevel = 0;

	setLevel(0);
}

GameFrame::~GameFrame()
{
	
	delete mGameObjectFactory;
	delete mTObjectFactory;
	delete mGameCharFactory;

	for(short i=0; i<mColMapSizeX; i++) 
	{
		delete mColMap[i];
	}
	delete[] mColMap;

	if(mCustomerArray != NULL)
	{
		for(int i=0; i<mCustomerArraySize; i++)
		{
			delete mCustomerArray[i];
		}
		delete[] mCustomerArray;
	}

	delete mObjectOven1;
	delete mObjectOven2;
	delete mObjectOven3;
	delete mObjectOven4;

	delete mObjectPrepTable1;
	delete mObjectPrepTable2;
	delete mObjectPrepTable3;
	delete mObjectPrepTable4;

	delete mObjectFridge;
	delete mObjectTrash;

	delete mCharLorenzo;

	delete mTray;

	delete mGameSpace;
	delete mGameUI;
}

void GameFrame::Draw(Graphics* g)
{
	g->SetColor(Color(63,127,127));
	g->FillRect(0,0,mWidth,mHeight);

	g->SetColor(Color(63, 127, 127, 95));
	g->mColorizeImages=true;
	g->DrawImage(pApp->mGrayLogoImg, -50, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 50, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 150, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 250, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 350, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 450, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 550, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 650, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 750, -25, 100, 100);

	g->DrawImage(pApp->mGrayLogoImg, -50, 75, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 750, 75, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, -50, 175, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 750, 175, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, -50, 275, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 750, 275, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, -50, 375, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 750, 375, 100, 100);

	g->DrawImage(pApp->mGrayLogoImg, -50, 475, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 50, 475, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 150, 475, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 250, 475, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 350, 475, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 450, 475, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 550, 475, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 650, 475, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 750, 475, 100, 100);
	
	
	g->mColorizeImages=false;
}

void GameFrame::Update()
{
	Widget::Update();

	if(!mPaused)
	{
		if(mUpdateCnt%100 == 0)
		{
			if(!pApp->mMusicInterface->IsPlaying(mPlayingSongNumber))
			{
				mPlayingSongNumber--;
				if(mPlayingSongNumber < 0)
				{
					mPlayingSongNumber = 4;
				}
				
				pApp->mMusicInterface->PlayMusic(mPlayingSongNumber);
			}
		}
	}

	if(!mPaused && !mGamePaused)
	{
		if(mShutdown == true)
		{
			pApp->ShowFrame(FrameFactory::MAINMENU);
			return;
		}

		if(mCountdown == 1)
		{
			generateCustomers();

			int spawnspeed = 2500 - mReputation*(mLevel+1)/10.0f;
			if (spawnspeed < 100) spawnspeed = 100;

			mCountdown = spawnspeed + Rand()%201;
			
		}
		else if(mCountdown > 1)
		{
			mCountdown--;
		}

		if(mCharLorenzo != NULL)
		{
			if(mCharLorenzo->mMoving)
			{
				if((mUpdateCnt - mCharLorenzo->mMovingStartTime) == 1)
				{
					MoveGameChar(mCharLorenzo, 3);
				}
			}
		}

		if(mTimer >= 0)
		{
			mTimer++;

			if(mTimer > mDayDuration)
			{
				int dailycost = (mLevel+1)*5;
				mFunds += -dailycost;

				pApp->mMusicInterface->StopAllMusic();

				if(mReputation < mDangerLine || mFunds <= 0)
				{
					GameOverlay* tempOverlay = new GameOverlay(pApp, this, -1);
					tempOverlay->mDisplayString[1] = "You Failed on Level " + StrFormat("%d", mLevel) + "!";
					tempOverlay->mDisplayStringColor[1] = Color(255,255,255);
					tempOverlay->mDisplayString[2] = "Better Luck Next Time!";
					tempOverlay->mDisplayStringColor[2] = Color(255,255,255);
					tempOverlay->mDisplayString[3] = "Press SpaceBar to return to the Main Menu!";
					tempOverlay->mDisplayStringColor[3] = Color(255,255,255);
					mWidgetManager->AddWidget(tempOverlay);
					mWidgetManager->SetFocus(tempOverlay);
					mPaused = true;

					pApp->PlaySample(GameApp::SFX_CrowdOoh);
					
					pSelectedChar->mMoving = false;
					mShutdown = true;

					//break;
				}
				else if(mReputation >= mGoalLine)
				{
					setLevel(mLevel+1);

					GameOverlay* tempOverlay = new GameOverlay(pApp, this, -1);
					tempOverlay->mDisplayString[0] = "Level Up!";
					tempOverlay->mDisplayStringColor[0] = Color(255,255,255);
					tempOverlay->mDisplayString[1] = "You are now at Level "+ StrFormat("%d", mLevel) + "!";
					tempOverlay->mDisplayStringColor[1] = Color(255,255,255);
					tempOverlay->mDisplayString[2] = "Keep up the Good Work!";
					tempOverlay->mDisplayStringColor[2] = Color(255,255,255);
					tempOverlay->mDisplayString[3] = "Press SpaceBar to Continue!";
					tempOverlay->mDisplayStringColor[3] = Color(255,255,255);
					mWidgetManager->AddWidget(tempOverlay);
					mWidgetManager->SetFocus(tempOverlay);
					mPaused = true;

					pApp->PlaySample(GameApp::SFX_Clap);
					/*
					for(int i=0; i<mTableArraySize; i++)
					{
						mTableArray[i]->setState(ObjectTable::EMPTY);
					}
					*/
					pSelectedChar->mMoving = false;
					
					mCountdown = 100 + Rand()%201;

					int dailycost = mLevel*5;
					PopupMsg* popup = new PopupMsg(pApp, this, 250);
					popup->Center(mGameUI->mPizzaTimer->mX+100, mGameUI->mPizzaTimer->mY+25);
					
					popup->mStringColor1= Color(191,0,191);
					popup->mDisplayString1 = StrFormat("- %d", dailycost);
					popup->mImageColor1 = Color(191,0,0);
					popup->pDisplayImage1 = pApp->mEuroImg;
					mWidgetManager->AddWidget(popup);
				}
				else
				{
					mTimer = 0;

					GameOverlay* tempOverlay = new GameOverlay(pApp, this, -1);
					tempOverlay->mDisplayString[1] = "You did not Reach the Goal in Time!";
					tempOverlay->mDisplayStringColor[1] = Color(255,255,255);
					tempOverlay->mDisplayString[2] = "Danger Zone Extended!";
					tempOverlay->mDisplayStringColor[2] = Color(255,255,255);
					tempOverlay->mDisplayString[3] = "Press SpaceBar to Continue!";
					tempOverlay->mDisplayStringColor[3] = Color(255,255,255);
					mWidgetManager->AddWidget(tempOverlay);
					mWidgetManager->SetFocus(tempOverlay);
					mPaused = true;

					mDangerLine += 25 + mLevel*10;

					pSelectedChar->mMoving = false;

					PopupMsg* popup = new PopupMsg(pApp, this, 250);
					popup->Center(mGameUI->mPizzaTimer->mX+100, mGameUI->mPizzaTimer->mY+25);
					
					popup->mStringColor1= Color(191,0,191);
					popup->mDisplayString1 = StrFormat("- %d", dailycost);
					popup->mImageColor1 = Color(191,0,0);
					popup->pDisplayImage1 = pApp->mEuroImg;
					mWidgetManager->AddWidget(popup);
				}
			}
		}
	}
}

void GameFrame::AddedToManager(WidgetManager* theWidgetManager)
{
	Widget::AddedToManager(theWidgetManager);

	mGameSpace = new GameSpace(pApp, this);
	mGameSpace->Resize(mOffsetX, mOffsetY, mGridSize*mColMapSizeX, mGridSize*mColMapSizeY);
	mWidgetManager->AddWidget(mGameSpace);


	mCharLorenzo = mGameCharFactory->getChar(GameCharFactory::LORENZO);
	AssignGameChar(mCharLorenzo, GameSimpleNode(10, 4));
	mWidgetManager->AddWidget(mCharLorenzo);


	mObjectOven1 = mGameObjectFactory->getObject(GameObjectFactory::OVEN);
	AssignGameObject(mObjectOven1, GameSimpleNode(14, 3), GameSimpleNode(13, 3));
	mWidgetManager->AddWidget(mObjectOven1);

	mObjectOven2 = mGameObjectFactory->getObject(GameObjectFactory::OVEN);
	AssignGameObject(mObjectOven2, GameSimpleNode(14, 4), GameSimpleNode(13, 4));
	mWidgetManager->AddWidget(mObjectOven2);

	mObjectOven3 = mGameObjectFactory->getObject(GameObjectFactory::OVEN);
	AssignGameObject(mObjectOven3, GameSimpleNode(14, 5), GameSimpleNode(13, 5));
	mWidgetManager->AddWidget(mObjectOven3);

	mObjectOven4 = mGameObjectFactory->getObject(GameObjectFactory::OVEN);
	AssignGameObject(mObjectOven4, GameSimpleNode(14, 6), GameSimpleNode(13, 6));
	mWidgetManager->AddWidget(mObjectOven4);

	mObjectPrepTable1 = mGameObjectFactory->getObject(GameObjectFactory::PREPTABLE);
	AssignGameObject(mObjectPrepTable1, GameSimpleNode(12, 3), GameSimpleNode(12, 3));
	mWidgetManager->AddWidget(mObjectPrepTable1);

	mObjectPrepTable2 = mGameObjectFactory->getObject(GameObjectFactory::PREPTABLE);
	AssignGameObject(mObjectPrepTable2, GameSimpleNode(12, 4), GameSimpleNode(12, 4));
	mWidgetManager->AddWidget(mObjectPrepTable2);

	mObjectPrepTable3 = mGameObjectFactory->getObject(GameObjectFactory::PREPTABLE);
	AssignGameObject(mObjectPrepTable3, GameSimpleNode(12, 5), GameSimpleNode(12, 5));
	mWidgetManager->AddWidget(mObjectPrepTable3);

	mObjectPrepTable4 = mGameObjectFactory->getObject(GameObjectFactory::PREPTABLE);
	AssignGameObject(mObjectPrepTable4, GameSimpleNode(12, 6), GameSimpleNode(12, 6));
	mWidgetManager->AddWidget(mObjectPrepTable4);

	mObjectFridge = mGameObjectFactory->getObject(GameObjectFactory::FRIDGE);
	AssignGameObject(mObjectFridge, GameSimpleNode(13, 1), GameSimpleNode(13, 2));
	mWidgetManager->AddWidget(mObjectFridge);

	mObjectTrash = mGameObjectFactory->getObject(GameObjectFactory::TRASH);
	AssignGameObject(mObjectTrash, GameSimpleNode(14, 8), GameSimpleNode(14, 7));
	mWidgetManager->AddWidget(mObjectTrash);

	
	mTableArraySize = 4;
	mTableArray = new ObjectTable*[mTableArraySize];

	mTableArray[0] = new ObjectTable(pApp, this);
	mTableArray[0]->mName = "Table 1";
	AssignGameObject(mTableArray[0], GameSimpleNode(2, 2), GameSimpleNode(2, 2));
	mWidgetManager->AddWidget(mTableArray[0]);
	mTableArray[0]->Spawn(0);
	mTableArray[0]->Spawn(1);
	mTableArray[0]->Spawn(2);
	mTableArray[0]->Spawn(3);

	mTableArray[1] = new ObjectTable(pApp, this);
	mTableArray[1]->mName = "Table 2";
	AssignGameObject(mTableArray[1], GameSimpleNode(7, 2), GameSimpleNode(7, 2));
	mWidgetManager->AddWidget(mTableArray[1]);
	mTableArray[1]->Spawn(0);
	mTableArray[1]->Spawn(1);
	mTableArray[1]->Spawn(2);
	mTableArray[1]->Spawn(3);

	mTableArray[2] = new ObjectTable(pApp, this);
	mTableArray[2]->mName = "Table 3";
	AssignGameObject(mTableArray[2], GameSimpleNode(2, 7), GameSimpleNode(2, 7));
	mWidgetManager->AddWidget(mTableArray[2]);
	mTableArray[2]->Spawn(0);
	mTableArray[2]->Spawn(1);
	mTableArray[2]->Spawn(2);
	mTableArray[2]->Spawn(3);

	mTableArray[3] = new ObjectTable(pApp, this);
	mTableArray[3]->mName = "Table 4";
	AssignGameObject(mTableArray[3], GameSimpleNode(7, 7), GameSimpleNode(7, 7));
	mWidgetManager->AddWidget(mTableArray[3]);
	mTableArray[3]->Spawn(0);
	mTableArray[3]->Spawn(1);
	mTableArray[3]->Spawn(2);
	mTableArray[3]->Spawn(3);


	pSelectedChar = mCharLorenzo;
	pSelectedChar->mSelected = true;
	pSelectedChar->mDirection = 1;
	//pSelectedChar = NULL;
	
	mWidgetManager->AddWidget(pCursorTrayWidget);

	mGameUI = new GameUI(pApp, this);
	mWidgetManager->AddWidget(mGameUI);
	
	GameOverlay* tempOverlay = new GameOverlay(pApp, this, -1);
	tempOverlay->mDisplayString[0] = "Welcome to the Pizza Parlor!";
	tempOverlay->mDisplayStringColor[0] = Color(255,255,255);
	tempOverlay->mDisplayString[1] = "Pass 10 Levels to Win!";
	tempOverlay->mDisplayStringColor[1] = Color(255,255,255);
	tempOverlay->mDisplayString[2] = "Let the Pizza Serving Competition Begin!";
	tempOverlay->mDisplayStringColor[2] = Color(255,255,255);
	tempOverlay->mDisplayString[3] = "Press SpaceBar to Start!";
	tempOverlay->mDisplayStringColor[3] = Color(255,255,255);
	mWidgetManager->AddWidget(tempOverlay);
	mPaused = true;

	mWidgetManager->SetFocus(this);

	mWidgetManager->BringToFront(mGameUI);
	mWidgetManager->SetFocus(tempOverlay);

	sortDepth();

	mPlayingSongNumber = Rand()%5;
	//pApp->mMusicInterface->PlayMusic(mPlayingSongNumber);
	//pApp->mMusicInterface->FadeIn(mPlayingSongNumber,0,-0.002, true);
}

void GameFrame::RemovedFromManager(WidgetManager* theWidgetManager)
{
	mWidgetManager->RemoveWidget(pCursorTrayWidget);

	mWidgetManager->RemoveWidget(mCharLorenzo);
	
	for(int i=0; i<mTableArraySize; i++)
	{
		mWidgetManager->RemoveWidget(mTableArray[i]);
	}
	
	mWidgetManager->RemoveWidget(mObjectPrepTable1);
	mWidgetManager->RemoveWidget(mObjectPrepTable2);
	mWidgetManager->RemoveWidget(mObjectPrepTable3);
	mWidgetManager->RemoveWidget(mObjectPrepTable4);

	mWidgetManager->RemoveWidget(mObjectOven1);
	mWidgetManager->RemoveWidget(mObjectOven2);
	mWidgetManager->RemoveWidget(mObjectOven3);
	mWidgetManager->RemoveWidget(mObjectOven4);

	mWidgetManager->RemoveWidget(mObjectFridge);
	mWidgetManager->RemoveWidget(mObjectTrash);

	mWidgetManager->RemoveWidget(mGameUI);
	mWidgetManager->RemoveWidget(mGameSpace);

	mWidgetManager->RemoveWidget(mGameOverlay);

	Widget::RemovedFromManager(theWidgetManager);

	pApp->mMusicInterface->FadeOut(mPlayingSongNumber);
}

void GameFrame::MouseMove(int x, int y)
{
	mMouseX = x;
	mMouseY = y;

	pCursorTrayWidget->Move(mMouseX+2, mMouseY+2);

	MarkDirty();
}

void GameFrame::MouseUp(int x, int y, int theBtnNum, int theClickCount)
{

}

void GameFrame::MoveCharTo(GameObject* theObj)
{
	if(pSelectedChar != NULL)
	{
		UpdateCharPaths(pSelectedChar, theObj);
		MoveGameChar(pSelectedChar);
	}
}

GameSimpleNode GameFrame::convertMapNodeToGlobalCoord(GameSimpleNode theNode)
{
	int globalX = theNode.x*mGridSize + mOffsetX;
	int globalY = theNode.y*mGridSize + mOffsetY;
	
	return GameSimpleNode(globalX, globalY);
}

void GameFrame::generateCustomers()
{
	int numNewCustomers = 1;

	numNewCustomers = 1+Rand()%4;

	//check all empty tables in the game for space
	ObjectTable* foundTable = NULL;
	bool found = false;

	int numSearched = 0;
	int begin = Rand()%mTableArraySize;

	for(int i=begin; i<mTableArraySize; )
	{
		if(mTableArray[i]->mNumChairs - mTableArray[i]->mNumCustomers >= numNewCustomers)
		{
			found = true;

			foundTable = mTableArray[i];

			foundTable->mNumCustomers = numNewCustomers;
			foundTable->mNumChairs = 0;
		
			break;
		}

		i++;
		if(i >= mTableArraySize) i = 0;

		numSearched++;
		if(numSearched >= mTableArraySize) break;
	}

	if(found) //if a table is found
	{
		for(int i=0; i<numNewCustomers; i++)
		{
			Customer* newCustomer;
			switch( Rand()%5 )
			{
			case 0: newCustomer = new Customer(pApp, this, Customer::MALE_NORMAL1);
					break;
			case 1: newCustomer = new Customer(pApp, this, Customer::MALE_NORMAL2);
					break;
			case 2: newCustomer = new Customer(pApp, this, Customer::MALE_NORMAL3);
					break;
			case 3: newCustomer = new Customer(pApp, this, Customer::MALE_NORMAL4);
					break;
			case 4: newCustomer = new Customer(pApp, this, Customer::MALE_NORMAL5);
					break;
			}
			
			newCustomer->pTable = foundTable;

			addCustomer(newCustomer);

			//insert new customer into a chair at the table
			foundTable->addCustomer(newCustomer);

		}
	}

	
}

void GameFrame::addCustomer(Customer* theCustomer)
{
	Customer** tempArray = new Customer*[mCustomerArraySize];

	for(int i=0; i<mCustomerArraySize; i++)
	{
		tempArray[i] = mCustomerArray[i];
	}
	
	if(mCustomerArray != NULL)
	{
		delete mCustomerArray;
	}

	mCustomerArray = new Customer*[mCustomerArraySize+1];

	for(int i=0; i<mCustomerArraySize; i++)
	{
		mCustomerArray[i] = tempArray[i];
	}

	delete tempArray;

	mCustomerArray[mCustomerArraySize] = theCustomer;

	mCustomerArraySize++;
}

bool GameFrame::removeCustomer(Customer* theCustomer)
{
	int removeIndex = -1;
	
	for(int i=0; i<mCustomerArraySize; i++)
	{
		if(mCustomerArray[mCustomerArraySize-1-i] == theCustomer)
		{
			removeIndex = mCustomerArraySize-1-i;
		}
	}

	if(removeIndex != -1)
	{
		delete mCustomerArray[removeIndex];

		for(int i=removeIndex; i<mCustomerArraySize-1; i++)
		{
			mCustomerArray[i] = mCustomerArray[i+1];
		}

		mCustomerArraySize--;

		Customer** tempArray = new Customer*[mCustomerArraySize];

		for(int i=0; i<mCustomerArraySize; i++)
		{
			tempArray[i] = mCustomerArray[i];
		}
		
		if(mCustomerArray != NULL)
		{
			delete mCustomerArray;
		}

		mCustomerArray = new Customer*[mCustomerArraySize];

		for(int i=0; i<mCustomerArraySize; i++)
		{
			mCustomerArray[i] = tempArray[i];
		}

		delete tempArray;

		return true;
	}
	else
	{
		return false;
	}
}


void GameFrame::AssignGameObject(GameObject* theObj, GameSimpleNode theNode, GameSimpleNode theFNode)
{
	int gridsize = mGridSize;
	
	theObj->Resize(0, 0, gridsize, gridsize);
	
	GameSimpleNode globalCoord = convertMapNodeToGlobalCoord(theNode);
	theObj->Move(globalCoord.x, globalCoord.y);

	theObj->Assign(theNode, theFNode);
}

void GameFrame::AssignGameChar(GameChar* theChar, GameSimpleNode theNode)
{
	int gridsize = mGridSize;
	
	theChar->Resize(0, 0, gridsize, gridsize);
	
	GameSimpleNode globalCoord = convertMapNodeToGlobalCoord(theNode);
	theChar->Move(globalCoord.x, globalCoord.y);

	theChar->AutoAssign();
}

void GameFrame::MoveGameChar(GameChar* theChar, short t)
{	
	int gridsize = mGridSize;
	int distance=1;
	
	for(short i=0; i<t; i++)
	{
		bool empty = false;

		GameSimpleNode srcNode;
		GameSimpleNode destNode;
		
		srcNode = theChar->mNode;

		if (theChar->mPath.empty())
		{
			empty = true;
		}
		else
		{
			destNode = theChar->mPath.back();

			GameSimpleNode globalCoord = convertMapNodeToGlobalCoord(destNode);

			Rect destRect = Rect(globalCoord.x, globalCoord.y, gridsize, gridsize);
			Rect srcRect = Rect(theChar->mX, theChar->mY, gridsize, gridsize);

			//if (srcNode == destNode)
			if(destRect==srcRect)
			{
				theChar->mPath.pop_back();

				if (theChar->mPath.empty())
				{
					empty = true;
				}
				else
				{
					destNode = theChar->mPath.back();
				}
			}
			else
			{
				int fX = 0;
				int fY = 0;
				
				int dX = destRect.mX - srcRect.mX;
				if(dX > 0)
				{
					fX = distance;
					theChar->mDirection = 3;
				}
				else if(dX < 0)
				{
					fX = -distance;
					theChar->mDirection = 1;
				}

				int dY = 0;
				if(dX == 0)
				{
					int dY = destRect.mY - srcRect.mY;
					if(dY > 0)
					{
						fY = distance;
						theChar->mDirection = 0;
					}
					else if(dY < 0)
					{
						fY = -distance;
						theChar->mDirection = 2;
					}
				}

				theChar->Move(theChar->mX+fX,theChar->mY+fY);
				theChar->AutoAssign();
				theChar->mMoving = true;
				theChar->mMovingStartTime = mUpdateCnt;
			}
		}

		if(empty)
		{
			GameSimpleNode globalCoord = convertMapNodeToGlobalCoord(theChar->pDestObj->mNode);
			
			Rect destRect = Rect(globalCoord.x, globalCoord.y, gridsize, gridsize);
			Rect srcRect = Rect(theChar->mX, theChar->mY, gridsize, gridsize);
			
			int fX = 0;
			int fY = 0;
			
			int dX = destRect.mX - srcRect.mX;
			if(dX > 0)
			{
				theChar->mDirection = 3;
			}
			else if(dX < 0)
			{
				theChar->mDirection = 1;
			}

			int dY = destRect.mY - srcRect.mY;
			if(dY > 0)
			{
				if(dX > 0)
				{
					theChar->mDirection = 7;
				}
				else if(dX < 0)
				{
					theChar->mDirection = 4;
				}
				else
				{
					theChar->mDirection = 0;
				}
			}
			else if(dY < 0)
			{
				if(dX > 0)
				{
					theChar->mDirection = 6;
				}
				else if(dX < 0)
				{
					theChar->mDirection = 5;
				}
				else
				{
					theChar->mDirection = 2;
				}
			}

			
			theChar->mMoving = false;

			theChar->pDestObj->mUsed = true;
			theChar->pDestObj->MouseUp(0,0,0,1);

			i=t;
		}
	}

	MarkDirty();
}

bool GameFrame::UpdateCharPaths(GameChar* theChar, GameObject* theObj)
{
	/*
	if (theChar->mNode == theObj->mFNode)
	{
		return false;
	}
	*/

	if (theChar->pDestObj == theObj)
	{
		return false;
	}
	
	if(theChar->pDestObj != NULL)
	{
		theChar->pDestObj->mUsed = false;
	}
	theChar->pDestObj = theObj;

	std::vector<GameSimpleNode> path;

	GameSimpleNode srcNode = theChar->mNode;
	GameSimpleNode destNode = theChar->pDestObj->getClosestFNode(srcNode);

	path = AstarFindPath(srcNode, destNode, mColMap, mColMapSizeX, mColMapSizeY);

	if(!path.empty())
	{
		theChar->mPath = path;
		return false;
	}

	return true;
}

std::vector<GameSimpleNode> GameFrame::AstarFindPath(GameSimpleNode srcNode, GameSimpleNode destNode, bool** theColMap, int mapSizeX, int mapSizeY)
{
	int cost = 1;
	std::vector<GameSimpleNode> path;

	bool valid = true;

	if(    (srcNode.x >= 0 && srcNode.x < mapSizeX)
		&& (srcNode.y >= 0 && srcNode.y < mapSizeY)
		&& (destNode.x >= 0 && destNode.x < mapSizeX)
		&& (destNode.y >= 0 && destNode.y < mapSizeY)  )
	{
		if(theColMap[destNode.x][destNode.y] == true)
		{
			valid = false;
		}
	}
	else
	{
		valid = false;
	}

	// verify valid src and dest nodes before starting
	if(valid)
	{
		//closed set: vector containing processed nodes (including ones in the open set as well)
		std::vector<GameNode*> closed;

		//open set: priority queue containing nodes to be processed
		myPQueue open;

		//generate nodeMap
		GameNode*** nodeMap;
		nodeMap = new GameNode**[mapSizeX];
		for (short i=0; i<mapSizeX; i++)
		{
			nodeMap[i] = new GameNode*[mapSizeY];
			for (short j=0; j<mapSizeY; j++)
			{
				nodeMap[i][j] = new GameNode(i,j);
			}
		}

		//plot src node on nodeMap
		GameNode* src = nodeMap[srcNode.x][srcNode.y];

		//fill astar info into src node
		src->g = 0;
		src->h = findHeuristic(src->getSimpleNode(), destNode);
		src->f = src->g + src->h;
		src->pParent = NULL;

		//push src node into open set
		open.push(src);

		bool keepSearching = true;
		DWORD startTime = GetTickCount();

		//start main loop
		while (keepSearching)
		{
			//pop the next node from open set
			GameNode* n = open.pop();
	
			//if the node = dest node
			if(n->getSimpleNode() == destNode)
			{
				//generate path and break the loop
				GameNode* itr;
				itr = n;
				path.push_back(itr->getSimpleNode());

				while(itr->pParent != NULL)
				{
					itr = itr->pParent;
					path.push_back(itr->getSimpleNode());
				}

				keepSearching = false;
				break;
			}

			//generate valid neighbour nodes Array
			GameNode** neighboursArr;

			short num_neighbours = 4;
			
			GameNode neighbour[4];
			neighbour[0].x = n->x-1;
			neighbour[0].y = n->y;
			neighbour[1].x = n->x;
			neighbour[1].y = n->y-1;
			neighbour[2].x = n->x+1;
			neighbour[2].y = n->y;
			neighbour[3].x = n->x;
			neighbour[3].y = n->y+1;

			bool pick_neighbour[4];
			for(short i=0; i<4; i++)
				pick_neighbour[i] = true;

			for(short i=0; i<4; i++)
			{
				if(neighbour[i].x >= mapSizeX || neighbour[i].x < 0
					|| neighbour[i].y >= mapSizeY || neighbour[i].y < 0
					|| theColMap[neighbour[i].x][neighbour[i].y] != 0)
				{
					num_neighbours--;
					pick_neighbour[i] = false;
				}
			}

			neighboursArr = new GameNode*[num_neighbours];
			short temp = 0;

			for(short i=0; i<4; i++)
			{
				if(pick_neighbour[i]==true)
				{
					neighboursArr[temp] = nodeMap[neighbour[i].x][neighbour[i].y];
					temp++;
				}
			}

			//for each neighbour node
			for(short i=0; i<num_neighbours; i++)
			{
				int newg = n->g + cost;
				GameNode* m = neighboursArr[i];

				bool visited = false;

				unsigned short closedSize = closed.size();				
				for(unsigned short j=0; j<closedSize; j++)
				{
					if(m->EqualTo(closed.at(closedSize-1-j)))
					{
						visited = true;
						break;
					}
				}

				//if this neightbour node does not exist in the closed set
				if(!visited)
				{
					//push this neighbour node into the open set
					m->g = newg;
					m->h = findHeuristic(m->getSimpleNode(), destNode);
					m->f = m->g + m->h;
					m->pParent = n;
					open.push(m);
					open.sortDec();
				}
				else
				{
					//if this neighbour node's new g is less than its previous one
					if(newg < m->g)
					{
						//update the g
						m->pParent = n;
						m->g = newg;
						m->f = m->g + m->h;

						//sort the open set
						open.sortDec();
					}
				}
			}

			//finished processing neighbour nodes
			delete neighboursArr;

			//push this node into the closed set
			closed.push_back(n);

			//stop searching if it takes too long
			if(GetTickCount() - startTime > 200)
			{
				keepSearching = false;
			}
		}
		//end main loop

		for(short i=0; i<mapSizeX; i++)
		{
			for(short j=0; j<mapSizeY; j++)
			{
				delete nodeMap[i][j];
			}
			delete nodeMap[i];
		}
		delete nodeMap;

		path.pop_back();
	}

	//return path
	return path;
}

int GameFrame::findHeuristic(GameSimpleNode srcNode, GameSimpleNode destNode)
{
	//using Area as our heuristic
	
	int h = 0;

	int dX = destNode.x - srcNode.x;
	int dY = destNode.y - srcNode.y;

	if(dX < 0) {dX = dX * -1;}
	if(dY < 0) {dY = dY * -1;}

	h =  dX + dY;

	return h;
}

void GameFrame::sortDepth()
{
	/*
	for(int i=0; i<mColMapSizeX; i++)
	{
		for(int j=0; j<mColMapSizeY; j++)
		{
			int x,y;
			Widget* w = mWidgetManager->GetWidgetAt(mOffsetX+i,mOffsetY+j, &x,&y);

			mWidgetManager->BringToFront(w);
		}
	}
	*/
	
	
	mWidgetManager->BringToFront(this->mGameOverlay);
	mWidgetManager->BringToFront(this->mGameUI);
	mWidgetManager->BringToFront(this->mGameUI->mPizzaTimer);
	mWidgetManager->BringToFront(this->pCursorTrayWidget);

}

void GameFrame::TogglePauseGame(bool forcePaused)
{	
	if(forcePaused && !mGamePaused)
	{
		mGameOverlay = new GameOverlay(pApp, this, -2);
		mGameOverlay->mDisplayString[4] = "PAUSED";
		mGameOverlay->mDisplayStringColor[4] = Color(255,255,255);
		mWidgetManager->AddWidget(mGameOverlay);

		sortDepth();

		mGameUI->TogglePauseGame(true);
		mGamePaused = true;
	}
	else if(!forcePaused && mGamePaused)
	{
		if(mPaused == true)
		{
			mWidgetManager->RemoveWidget(mGameOverlay);
			mPaused = true;
		}
		else
		{
			mWidgetManager->RemoveWidget(mGameOverlay);
		};
		
		mGameUI->TogglePauseGame(false);
		mGamePaused = false;
	}

	
	MarkDirty();

}

void GameFrame::TogglePauseGame()
{	
	if(!mGamePaused)
	{
		TogglePauseGame(true);
	}
	else
	{
		TogglePauseGame(false);
	}

}

void GameFrame::setLevel(int newLevel)
{
	if(newLevel < 0) newLevel = 0;
	mLevel = newLevel;

	switch(newLevel)
	{
	case 0: mGoalLine = 120;
			mDangerLine = 0;
			mMaxReputation = 200;

			mFunds = 100;
			mDayDuration = 12000;
			break;

	case 10:	{	
					GameOverlay* tempOverlay = new GameOverlay(pApp, this, -1);
					tempOverlay->mDisplayString[0] = "Congratulation! You Won!";
					tempOverlay->mDisplayStringColor[0] = Color(255,255,255);
					tempOverlay->mDisplayString[1] = "You earned " + StrFormat("%d", mFunds) + " Euros!";
					tempOverlay->mDisplayStringColor[1] = Color(255,255,255);
					tempOverlay->mDisplayString[2] = "Press SpaceBar to return to the Main Menu!";
					tempOverlay->mDisplayStringColor[2] = Color(255,255,255);
					mWidgetManager->AddWidget(tempOverlay);
					mWidgetManager->SetFocus(tempOverlay);
					mPaused = true;
					
					mShutdown = true;
				}
				break;

	default:	mGoalLine += 120 + mLevel*10;
				mFunds += mLevel*100;
			
				mMaxReputation += 200;

				mDayDuration = 12000;
				break;

	}

	mDangerLine += 10 + mLevel*10;
	mTimer = 0;

	//mCountdown = 200 + Rand()%201;
}

void GameFrame::KeyUp(KeyCode theKey)
{
	if(theKey == KeyCode::KEYCODE_SPACE)
	{ 
		TogglePauseGame();
	}
	
	if(theKey == KeyCode::KEYCODE_ADD)
	{
		mReputation += 100;
		MarkDirty();
	}

	if(theKey == KeyCode::KEYCODE_SUBTRACT)
	{
		mReputation -= 100;
		MarkDirty();
	}

	if(theKey == KeyCode::KEYCODE_MULTIPLY)
	{ 
		setLevel(mLevel+1);
		MarkDirty();
	}

	if(theKey == KeyCode::KEYCODE_DIVIDE)
	{ 
		setLevel(mLevel-1);
		MarkDirty();
	}
}