#ifndef __GAMEFRAME_H__
#define __GAMEFRAME_H__

#include "../../../SexyAppFramework/Widget.h"

namespace Sexy
{
	class Graphics;
	class WidgetManager;

	class GameApp;

	class GameSpace;
	class GameUI;
	class GameOverlay;

	class GameObjectFactory;
	class GameObject;
	class ObjectTable;

	class TObjectFactory;
	class TObject;

	class GameCharFactory;
	class GameChar;

	class GameSimpleNode;
	class GameNode;

	class Customer;

	class Tray;
	class Widget;

class GameFrame : public Widget
{
	private:
		GameApp* pApp;

		int mCountdown;
		int mPlayingSongNumber;
		
	public:
		GameObjectFactory*	mGameObjectFactory;
		TObjectFactory*		mTObjectFactory;
		GameCharFactory*	mGameCharFactory;

		Tray* mTray;
		Widget* pCursorTrayWidget;

		GameChar* pSelectedChar;

		bool** mColMap;
		short mColMapSizeX;
		short mColMapSizeY;

		int mOffsetX;
		int mOffsetY;

		int mGridSize;

		GameSpace*		mGameSpace;
		GameUI*			mGameUI;
		GameOverlay*	mGameOverlay;

		GameObject*		mObjectOven1;
		GameObject*		mObjectOven2;
		GameObject*		mObjectOven3;
		GameObject*		mObjectOven4;

		GameObject*		mObjectPrepTable1;
		GameObject*		mObjectPrepTable2;
		GameObject*		mObjectPrepTable3;
		GameObject*		mObjectPrepTable4;

		GameObject*		mObjectFridge;
		GameObject*		mObjectTrash;

		GameChar*		mCharLorenzo;

		int mCustomerArraySize;
		Customer** mCustomerArray;

		int mTableArraySize;
		ObjectTable** mTableArray;

		int mMouseX;
		int mMouseY;

		double mTimer;
		int mFunds;
		int mReputation;

		bool mPaused;
		bool mGamePaused;

		int mGoalLine;
		int mDangerLine;
		int mMaxReputation;

		int mDayDuration;
		int mLevel;

		bool mShutdown;

	public:
		GameFrame(GameApp* theApp);
		virtual ~GameFrame();	

		virtual void Draw(Graphics* g);
		virtual void Update();

		virtual void AddedToManager(WidgetManager* theWidgetManager);
		virtual void RemovedFromManager(WidgetManager* theWidgetManager);

		virtual void MouseMove(int x, int y);
		virtual void MouseUp(int x, int y, int theBtnNum, int theClickCount);

		virtual void MoveCharTo(GameObject* theObj);

		GameSimpleNode convertMapNodeToGlobalCoord(GameSimpleNode theNode);

		void generateCustomers();
		void addCustomer(Customer* theCustomer);
		bool removeCustomer(Customer* theCustomer);

		void AssignGameObject(GameObject* theObj, GameSimpleNode theNode, GameSimpleNode theFNode);
		void AssignGameChar(GameChar* theChar, GameSimpleNode theNode);

		void MoveGameChar(GameChar* theChar, short t=1);
		bool UpdateCharPaths(GameChar* theChar, GameObject* theObj);

		std::vector<GameSimpleNode> AstarFindPath(GameSimpleNode srcNode, GameSimpleNode destNode, bool** theColMap, int mapSizeX, int mapSizeY); 
		int findHeuristic(GameSimpleNode srcNode, GameSimpleNode destNode);

		void sortDepth();

		void TogglePauseGame(bool forcePaused);
		void TogglePauseGame();

		void setLevel(int newLevel);

		virtual void KeyUp(KeyCode theKey);

};

}

#endif	// __GAMEFRAME_H__