#include "GameObjectFactory.h"
#include "GameObject.h"
#include "GameApp.h"
#include "GameFrame.h"

#include "Tray.h"
#include "TObject.h"
#include "TObjectFactory.h"

#include "GameNode.h"

#include "../../../SexyAppFramework/WidgetManager.h"

using namespace Sexy;

GameObject::GameObject(GameApp* theApp, GameFrame* theFrame)
{
	pApp = theApp;
	pFrame = theFrame;

	mType = -1;
	mName = "";
	pImg = NULL;

	mActive = true;
	mSelectable = true;
	mUsed = false;
	mHighlightColor = Color(255, 255, 255, 63);
	mNormalColor = Color(0, 0, 0, 0);
	mColor = mNormalColor;

	mConsumer = false;
	mDispenser = false;

	//Resize(0, 480, 800, 120);
}

GameObject::~GameObject()
{
}

void GameObject::Draw(Graphics* g)
{
	g->SetColor(mColor);
	g->FillRect(0, 0, mWidth, mHeight);
}

void GameObject::Update()
{
	Widget::Update();
}

void GameObject::AddedToManager(WidgetManager* theWidgetManager)
{
	Widget::AddedToManager(theWidgetManager);
}

void GameObject::RemovedFromManager(WidgetManager* theWidgetManager)
{
	Widget::RemovedFromManager(theWidgetManager);
}

TObject* GameObject::Dispense()
{
	return false;
}

bool GameObject::Consume(Tray* theTray)
{
	return false;
}

void GameObject::Spawn(int i)
{
}

void GameObject::Delivered(short id)
{

}


void GameObject::MouseMove(int x, int y)
{
	pFrame->MouseMove(x+mX,y+mY);
}

void GameObject::MouseEnter()
{
	if(mSelectable)
	{
		mColor = mHighlightColor;
		pFrame->MarkDirty();
	}
}

void GameObject::MouseLeave()
{
	if(mSelectable)
	{
		mColor = mNormalColor;
		pFrame->MarkDirty();
	}
}

void GameObject::MouseDown(int x, int y, int theBtnNum, int theClickCount)
{
}

void GameObject::MouseUp(int x, int y, int theBtnNum, int theClickCount)
{
	if(theBtnNum == 0)
	{
		if(mUsed)
		{
			if(mActive)
			{
				bool ItemConsumed = false;
				
				//if(!pFrame->mTray->isEmpty())
				{
					if(mConsumer)
					{
						ItemConsumed = Consume(pFrame->mTray);
					}
				}

				if(!ItemConsumed)
				{
					if(pFrame->mTray->hasSpace())
					{
						if(mDispenser)
						{
							pFrame->mTray->addItem(Dispense());
						}
					}
					else
					{
						pApp->PlaySample(GameApp::SFX_Thud);
					}
					
				}
				
				
			}
		}
		else
		{
			pFrame->MoveCharTo(this);
		}
	}
	else if(theBtnNum == 1)
	{
		Consume(NULL);
	}
}

void GameObject::Assign(GameSimpleNode theNode, GameSimpleNode theFNode)
{
	mNode = theNode;
	mFNode = theFNode;

	pFrame->mColMap[theNode.x][theNode.y] = 1;
}

GameSimpleNode GameObject::getClosestFNode(GameSimpleNode theNode)
{
	GameSimpleNode* node;
	int numNodes = 0;

	if(mFNode == mNode)
	{
		GameSimpleNode tempNode[4];
		
		tempNode[0] = GameSimpleNode(mNode.x, mNode.y-1);
		tempNode[1] = GameSimpleNode(mNode.x+1, mNode.y);
		tempNode[2] = GameSimpleNode(mNode.x, mNode.y+1);
		tempNode[3] = GameSimpleNode(mNode.x-1, mNode.y);
		
		for(int i=0; i<4; i++)
		{
			if(		tempNode[i].x >= 0 && tempNode[i].x < pFrame->mColMapSizeX
				&&	tempNode[i].y >= 0 && tempNode[i].y < pFrame->mColMapSizeY )
			{
				numNodes++;
			}
			else
			{
				tempNode[i] = mNode;
			}
		}

		node = new GameSimpleNode[numNodes];
		int f=0;

		for(int i=0; i<4; i++)
		{
			if(!(tempNode[i] == mNode))
			{
				node[f++] = tempNode[i];
			}
		}
	}
	else
	{
		numNodes = 1;
		node = new GameSimpleNode[numNodes];

		node[0] = mFNode;
	}

	int dx;
	int dy;

	int* dist = new int[numNodes];

	for(int i=0; i<numNodes; i++)
	{
		if(pFrame->mColMap[node[i].x][node[i].y] != 0)
		{
			dist[i] = 100;

		}
		else
		{
			dx = node[i].x - theNode.x;
			dy = node[i].y - theNode.y;

			if(dx < 0) dx *= -1;
			if(dy < 0) dy *= -1;

			dist[i] = dx + dy;
		}
	}

	int minIndex = -1;
	int min = 100;
		
	for(int i=0; i<numNodes; i++)
	{
		if(dist[i] < min)
		{
			min = dist[i];
			minIndex = i;
		}
	}

	GameSimpleNode returnNode = mNode;
		
	if(minIndex >= 0)
	{
		returnNode = GameSimpleNode(node[minIndex].x, node[minIndex].y);
	}

	delete[] node;
	delete[] dist;
	
	return returnNode;
}
