#include "GameOverlay.h"
#include "GameApp.h"

#include "GameFrame.h"

#include "TObjectFactory.h"
#include "TObject.h"

#include "../../../SexyAppFramework/WidgetManager.h"
#include "../../../SexyAppFramework/SysFont.h"
#include "../../../SexyAppFramework/ImageFont.h"

using namespace Sexy;

GameOverlay::GameOverlay(GameApp* theApp, GameFrame* theFrame, int theTimer)
{
	pApp = theApp;
	pFrame = theFrame;
	mTimer = theTimer;

	Resize(0, 0, pApp->mWidth, pApp->mHeight);
}

GameOverlay::~GameOverlay()
{
}

void GameOverlay::Draw(Graphics* g)
{
	g->SetColor(Color(0,0,0,127));
	g->FillRect(0,0,mWidth,mHeight);

	for(int i=0; i<5; i++)
	{
		g->SetColor(mDisplayStringColor[i]);
		g->SetFont(pApp->mBigSysFont);
		g->WriteString(mDisplayString[i], 0 , mHeight/2 - 100 + i*50, mWidth, 0);
	}
}

void GameOverlay::Update()
{
	Widget::Update();

	if(!pFrame->mGamePaused)
	{
		if(mTimer > 0)
		{
			mTimer--;
		}
		else if(mTimer == 0)
		{
			mWidgetManager->RemoveWidget(this);
			pFrame->MarkDirty();
		}
	}
}

void GameOverlay::AddedToManager(WidgetManager* theWidgetManager)
{
	Widget::AddedToManager(theWidgetManager);

	//mWidgetManager->SetFocus(this);
	//pFrame->mPaused = true;
}

void GameOverlay::RemovedFromManager(WidgetManager* theWidgetManager)
{
	pFrame->mPaused = false;

	pApp->SafeDeleteWidget(this);

	Widget::RemovedFromManager(theWidgetManager);
}

void GameOverlay::KeyUp(KeyCode theKey)
{
	if(mTimer == -1)
	{
		if(theKey == KEYCODE_SPACE)
		{
			pFrame->MarkDirty();
			mWidgetManager->SetFocus(pFrame);
			mWidgetManager->RemoveWidget(this);
		}
	}
}