#include "GameUI.h"
#include "GameApp.h"
#include "GameOverlay.h"
#include "GameFrame.h"
#include "HappinessBar.h"
#include "PizzaTimer.h"

#include "../../../SexyAppFramework/WidgetManager.h"
#include "../../../SexyAppFramework/Font.h"
#include "../../../SexyAppFramework/SysFont.h"
#include "../../../SexyAppFramework/ImageFont.h"
#include "../../../SexyAppFramework/ButtonWidget.h"

#include "../../../SexyAppFramework/SoundManager.h"
#include "../../../SexyAppFramework/BassMusicInterface.h"

using namespace Sexy;

GameUI::GameUI(GameApp* theApp, GameFrame* theFrame)
{
	pApp = theApp;
	pFrame = theFrame;

	Resize(0, 550, 800, 50);

	mBtnMainMenu = new ButtonWidget(BTN_MainMenu, this);
	mBtnPauseGame = new ButtonWidget(BTN_PauseGame, this);
	mBtnToggleFullScreen = new ButtonWidget(BTN_ToggleFullScreen, this);
	mBtnMute = new ButtonWidget(BTN_Mute, this);

	mHappinessBar = new HappinessBar(pApp, pFrame);
	mPizzaTimer = new PizzaTimer(pApp, pFrame);
}

GameUI::~GameUI()
{
	delete mPizzaTimer;
	delete mHappinessBar;

	delete mBtnMainMenu;
	delete mBtnPauseGame;
	delete mBtnToggleFullScreen;
	delete mBtnMute;

}

void GameUI::Draw(Graphics* g)
{
	g->SetColor(Color(127, 0, 0));
	g->FillRect(0, 0, mWidth, mHeight);

	g->SetColor(Color(127, 190, 190, 95));
	g->mColorizeImages=true;
	g->DrawImage(pApp->mGrayLogoImg, -50, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 50, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 150, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 250, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 350, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 450, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 550, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 650, -25, 100, 100);
	g->DrawImage(pApp->mGrayLogoImg, 750, -25, 100, 100);
	g->mColorizeImages=false;

	g->SetColor(Color(255,255,255));
	g->SetFont(pApp->mSysFont);

	//g->WriteString("Timer", 590, 0+pApp->mSysFont->GetHeight()-3, 100, 0);
	//g->WriteString(StrFormat("%d", (int)pFrame->mTimer/100), 695, 0+pApp->mSysFont->GetHeight()-3, 100, 0);

	//g->SetColor(Color(127,127,127));
	//g->FillRect(400, 5, 40, 40);
	
	g->SetColor(Color(127,127,127));
	g->FillRect(445, 5, 120, 40);
	g->SetColor(Color(0,0,0));
	g->DrawRect(445, 5, 119, 39);

	g->mColorizeImages = true;
	g->SetColor(Color(255,255,0));
	g->DrawImage(pApp->mEuroImg, 400, 5);
	g->mColorizeImages = true;
	g->SetColor(Color(255,255,255));
	g->SetFont(pApp->mBigSysFont);
	g->WriteString(StrFormat("%d", pFrame->mFunds), 455, 15+pApp->mSysFont->GetHeight()-3, 100, 1);

	//g->WriteString("Reputation", 400, 3+pApp->mSysFont->GetHeight()-3, 90, 0);
	//g->WriteString(StrFormat("%d", pFrame->mReputation), 400, 25+pApp->mSysFont->GetHeight()-3, 90, 0);
}

void GameUI::Update()
{
	Widget::Update();
}

void GameUI::AddedToManager(WidgetManager* theWidgetManager)
{
	Widget::AddedToManager(theWidgetManager);

	
	mBtnMainMenu->Resize(730, 22, 25, 25);
	mBtnMainMenu->SetColor(ButtonWidget::COLOR_LABEL, Color(0, 0, 0));
	mBtnMainMenu->SetColor(ButtonWidget::COLOR_LABEL_HILITE, Color(255, 0, 0));
	mBtnMainMenu->mButtonImage = pApp->mMainMenuBtnDown;
	mBtnMainMenu->mDoFinger = true;
	AddWidget(mBtnMainMenu);

	
	mBtnPauseGame->Resize(695, 22, 25, 25);
	mBtnPauseGame->SetColor(ButtonWidget::COLOR_LABEL, Color(0, 0, 0));
	mBtnPauseGame->SetColor(ButtonWidget::COLOR_LABEL_HILITE, Color(255, 0, 0));
	mBtnPauseGame->mButtonImage = pApp->mPauseBtnDown;
	mBtnPauseGame->mDoFinger = true;
	AddWidget(mBtnPauseGame);

	
	mBtnToggleFullScreen->Resize(660, 22, 25, 25);
	mBtnToggleFullScreen->SetColor(ButtonWidget::COLOR_LABEL, Color(0, 0, 0));
	mBtnToggleFullScreen->SetColor(ButtonWidget::COLOR_LABEL_HILITE, Color(255, 0, 0));
	if(gSexyAppBase->mIsWindowed)
	{
		mBtnToggleFullScreen->mButtonImage = pApp->mFullscreenBtnDown;
	}
	else
	{
		mBtnToggleFullScreen->mButtonImage = pApp->mWindowedBtnDown;
	}
	mBtnToggleFullScreen->mDoFinger = true;
	AddWidget(mBtnToggleFullScreen);

	mBtnMute->Resize(625, 22, 25, 25);
	mBtnMute->SetColor(ButtonWidget::COLOR_LABEL, Color(0, 0, 0));
	mBtnMute->SetColor(ButtonWidget::COLOR_LABEL_HILITE, Color(255, 0, 0));
	if(pApp->mSfxVolume != 0)
	{
		mBtnMute->mButtonImage = pApp->mMuteBtnDown;
	}
	else
	{
		mBtnMute->mButtonImage = pApp->mUnmuteBtnDown;
	}
	mBtnMute->mDoFinger = true;
	AddWidget(mBtnMute);

	mHappinessBar->Move(5,5);
	AddWidget(mHappinessBar);

	mPizzaTimer->Move(mX+590,mY-75);
	mWidgetManager->AddWidget(mPizzaTimer);
	mWidgetManager->BringToFront(mPizzaTimer);

}

void GameUI::RemovedFromManager(WidgetManager* theWidgetManager)
{
	RemoveWidget(mBtnMainMenu);
	RemoveWidget(mBtnPauseGame);
	RemoveWidget(mBtnToggleFullScreen);
	RemoveWidget(mBtnMute);

	RemoveWidget(mHappinessBar);
	RemoveWidget(mPizzaTimer);
	
	Widget::RemovedFromManager(theWidgetManager);
}

void GameUI::MouseMove(int x, int y)
{
	pFrame->MouseMove(x+mX,y+mY);
}

void GameUI::ButtonDepress(int theId)
{
	switch(theId)
	{
	case BTN_MainMenu:	
						pApp->PlaySample(GameApp::SFX_Click);
						pApp->ShowFrame(FrameFactory::MAINMENU); 
						break;

	case BTN_PauseGame:	
						pApp->PlaySample(GameApp::SFX_Click);
						pFrame->TogglePauseGame();
						break;

	case BTN_ToggleFullScreen:  
								pApp->PlaySample(GameApp::SFX_Click);
								if(gSexyAppBase->mIsWindowed)
								{
									gSexyAppBase->SwitchScreenMode(false);
									mBtnToggleFullScreen->mButtonImage = pApp->mWindowedBtnDown;
								}
								else
								{
									gSexyAppBase->SwitchScreenMode(true);
									mBtnToggleFullScreen->mButtonImage = pApp->mFullscreenBtnDown;
								}
								break;

								
	case BTN_Mute:				
								pApp->PlaySample(GameApp::SFX_Click);
								if(pApp->mSfxVolume != 0.00)
								{
									pApp->mSfxVolume = 0.00;
									pApp->mMusicVolume = 0.00;
									pApp->mSoundManager->SetVolume(0.00);
									pApp->mMusicInterface->SetVolume(0.00);
									mBtnMute->mButtonImage = pApp->mUnmuteBtnDown;
								}
								else
								{
									pApp->mSfxVolume = 0.50;
									pApp->mMusicVolume = 0.25;
									pApp->mSoundManager->SetVolume(0.50);
									pApp->mMusicInterface->SetVolume(0.25);
									mBtnMute->mButtonImage = pApp->mMuteBtnDown;
								}
								break;


	}
}

void GameUI::TogglePauseGame(bool forcePaused)
{
	if(forcePaused)
	{
		//mBtnPauseGame->mLabel = "Resume";
		mBtnPauseGame->mButtonImage = pApp->mResumeBtnDown;
	}
	else
	{
		//mBtnPauseGame->mLabel = "Pause";
		mBtnPauseGame->mButtonImage = pApp->mPauseBtnDown;
	}
}
