#include "GameObjectFactory.h"
#include "GameObject.h"
#include "ObjectFridge.h"
#include "GameApp.h"
#include "GameFrame.h"
#include "Selector.h"

#include "PopupMsg.h"

#include "Tray.h"
#include "TObject.h"
#include "TObjectFactory.h"

#include "../../../SexyAppFramework/WidgetManager.h"

using namespace Sexy;

ObjectFridge::ObjectFridge(GameApp* theApp, GameFrame* theFrame):GameObject(theApp, theFrame)
{
	mType = GameObjectFactory::FRIDGE;
	mName = "Fridge";

	pImg = pApp->mFridgeImg;

	mDispenser = true;
	mConsumer = false;

	mIngredientSelector = NULL;
	mSelectedIngredient = -1;

	mIngredientSelector = new Selector(pApp, pFrame, this);
}

ObjectFridge::~ObjectFridge()
{
}

void ObjectFridge::Draw(Graphics* g)
{
	//g->SetColor(Color(0, 0, 127));
	//g->FillRect(0, 0, mWidth, mHeight);

	g->DrawImage(pImg, 0, 0, mWidth, mHeight);
	
	g->SetColor(mColor);
	g->FillRect(0, 0, mWidth, mHeight);
}

void ObjectFridge::Update()
{
	Widget::Update();
}

void ObjectFridge::AddedToManager(WidgetManager* theWidgetManager)
{
	Widget::AddedToManager(theWidgetManager);

	mWidgetManager->AddWidget(mIngredientSelector);

	TObject* tempIngredient = pFrame->mTObjectFactory->getTObject(TObjectFactory::BASIL);
	mIngredientSelector->AddItem(tempIngredient->mType, tempIngredient->pImg, tempIngredient->mName, tempIngredient->mColor);
	delete tempIngredient;

	tempIngredient = pFrame->mTObjectFactory->getTObject(TObjectFactory::OREGANO);
	mIngredientSelector->AddItem(tempIngredient->mType, tempIngredient->pImg, tempIngredient->mName, tempIngredient->mColor);
	delete tempIngredient;
	
	tempIngredient = pFrame->mTObjectFactory->getTObject(TObjectFactory::GARLIC);
	mIngredientSelector->AddItem(tempIngredient->mType, tempIngredient->pImg, tempIngredient->mName, tempIngredient->mColor);
	delete tempIngredient;

	tempIngredient = pFrame->mTObjectFactory->getTObject(TObjectFactory::ONION);
	mIngredientSelector->AddItem(tempIngredient->mType, tempIngredient->pImg, tempIngredient->mName, tempIngredient->mColor);
	delete tempIngredient;

	tempIngredient = pFrame->mTObjectFactory->getTObject(TObjectFactory::ANCHOVY);
	mIngredientSelector->AddItem(tempIngredient->mType, tempIngredient->pImg, tempIngredient->mName, tempIngredient->mColor);
	delete tempIngredient;

	tempIngredient = pFrame->mTObjectFactory->getTObject(TObjectFactory::SAUSAGE);
	mIngredientSelector->AddItem(tempIngredient->mType, tempIngredient->pImg, tempIngredient->mName, tempIngredient->mColor);
	delete tempIngredient;	
	
}

void ObjectFridge::RemovedFromManager(WidgetManager* theWidgetManager)
{
	Widget::RemovedFromManager(theWidgetManager);
}

TObject* ObjectFridge::Dispense()
{
	TObject* retObj = NULL;

	if (pFrame->mFunds > 0)
	{
		retObj = pFrame->mTObjectFactory->getTObject(mSelectedIngredient);
		
		PopupMsg* popup = new PopupMsg(pApp, pFrame, 250);
		popup->Center(pFrame->mMouseX+5, mY-10);
		pFrame->mFunds += -1;
		popup->mStringColor1= Color(191,0,191);
		popup->mDisplayString1 = StrFormat("- 1");
		popup->mImageColor1 = Color(191,0,0);
		popup->pDisplayImage1 = pApp->mEuroImg;
		mWidgetManager->AddWidget(popup);
		

		pApp->PlaySample(GameApp::SFX_Click);
	}

	mSelectedIngredient = -1;

	if(retObj == NULL)
	{
		pApp->PlaySample(GameApp::SFX_Thud);
	}

	return retObj;
}

bool ObjectFridge::Consume(Tray* theTray)
{
	if(theTray != NULL)
		pApp->PlaySample(GameApp::SFX_Thud);

	
	return true;
}

void ObjectFridge::Assign(GameSimpleNode theNode, GameSimpleNode theFNode)
{
	mNode = theNode;
	mFNode = theFNode;

	pFrame->mColMap[theNode.x-1][theNode.y] = 1;
	pFrame->mColMap[theNode.x][theNode.y] = 1;
	pFrame->mColMap[theNode.x+1][theNode.y] = 1;

	Resize(mX-mWidth, mY, mWidth*3, mHeight);
}

GameSimpleNode ObjectFridge::getClosestFNode(GameSimpleNode theNode)
{
	GameSimpleNode* node;
	int numNodes = 0;

	if(mFNode.y != mNode.y)
	{
		GameSimpleNode tempNode[3];
		
		int multiplier = 1;
		if(mFNode.y < mNode.y)
		{
			multiplier = -1;
		}

		tempNode[0] = GameSimpleNode(mNode.x-1, mNode.y+1*multiplier);
		tempNode[1] = GameSimpleNode(mNode.x, mNode.y+1*multiplier);
		tempNode[2] = GameSimpleNode(mNode.x+1, mNode.y+1*multiplier);
		
		for(int i=0; i<3; i++)
		{
			if(		tempNode[i].x >= 0 && tempNode[i].x < pFrame->mColMapSizeX
				&&	tempNode[i].y >= 0 && tempNode[i].y < pFrame->mColMapSizeY )
			{
				numNodes++;
			}
			else
			{
				tempNode[i] = NULL;
			}
		}

		node = new GameSimpleNode[numNodes];
		int f=0;

		for(int i=0; i<3; i++)
		{
			if(!(tempNode[i] == NULL))
			{
				node[f++] = tempNode[i];
			}
		}
	}
	else
	{
		GameSimpleNode tempNode[6];

		tempNode[0] = GameSimpleNode(mNode.x-1, mNode.y-1);
		tempNode[1] = GameSimpleNode(mNode.x, mNode.y-1);
		tempNode[2] = GameSimpleNode(mNode.x+1, mNode.y-1);
		tempNode[3] = GameSimpleNode(mNode.x-1, mNode.y+1);
		tempNode[4] = GameSimpleNode(mNode.x, mNode.y+1);
		tempNode[5] = GameSimpleNode(mNode.x+1, mNode.y+1);
		
		for(int i=0; i<6; i++)
		{
			if(		tempNode[i].x >= 0 && tempNode[i].x < pFrame->mColMapSizeX
				&&	tempNode[i].y >= 0 && tempNode[i].y < pFrame->mColMapSizeY )
			{
				numNodes++;
			}
			else
			{
				tempNode[i] = NULL;
			}
		}

		node = new GameSimpleNode[numNodes];
		int f=0;

		for(int i=0; i<6; i++)
		{
			if(!(tempNode[i] == NULL))
			{
				node[f++] = tempNode[i];
			}
		}
	}

	int dx;
	int dy;

	int* dist = new int[numNodes];

	for(int i=0; i<numNodes; i++)
	{
		if(pFrame->mColMap[node[i].x][node[i].y] != 0)
		{
			dist[i] = 100;

		}
		else
		{
			dx = node[i].x - theNode.x;
			dy = node[i].y - theNode.y;

			if(dx < 0) dx *= -1;
			if(dy < 0) dy *= -1;

			dist[i] = dx + dy;
		}
	}

	int minIndex = -1;
	int min = 100;
		
	for(int i=0; i<numNodes; i++)
	{
		if(dist[i] < min)
		{
			min = dist[i];
			minIndex = i;
		}
	}

	GameSimpleNode returnNode = mNode;
		
	if(minIndex >= 0)
	{
		returnNode = GameSimpleNode(node[minIndex].x, node[minIndex].y);
	}

	delete[] node;
	delete[] dist;
	
	return returnNode;
}

void ObjectFridge::MouseEnter()
{
	if(mSelectable)
	{
		mIngredientSelector->ShowSelector(true);

		mColor = Color(255, 255, 255, 127);
		pFrame->MarkDirty();
	}
}

void ObjectFridge::MouseLeave()
{
	if(mSelectable)
	{
		mIngredientSelector->ShowSelector(false);

		mColor = Color(0, 0, 0, 0);
		pFrame->MarkDirty();
	}
}

void ObjectFridge::Delivered(short id)
{
	mSelectedIngredient = id;
}	

