#include "GameObjectFactory.h"
#include "GameObject.h"
#include "ObjectOrderTable.h"
#include "GameApp.h"
#include "GameFrame.h"

#include "TObject.h"
#include "TObjectFactory.h"

#include "../../../SexyAppFramework/WidgetManager.h"

using namespace Sexy;

ObjectOrderTable::ObjectOrderTable(GameApp* theApp, GameFrame* theFrame):GameObject(theApp, theFrame)
{
	mType = GameObjectFactory::ORDERTABLE;
	mName = "Order Table";

	//Resize(0, 0, 50, 50);

	mConsumer = true;
	mDispenser = false;

	mOrder = NULL;

	mPizzaContainerSize = 0;
	mPizzaContainer = NULL;

	mPizzaArraySize = 0;
	mPizzaFillArray = NULL;
	mPizzaDeliveryArray = NULL;

	mNumPizzaDelivered = 0;
	mNumPizzaFilled = 0;

	setState(EMPTY);
}

ObjectOrderTable::~ObjectOrderTable()
{
	releaseArrays();
}

void ObjectOrderTable::Draw(Graphics* g)
{
	g->SetColor(Color(0, 0, 127));
	g->FillRect(0, 0, mWidth, mHeight);
	
	g->SetColor(mColor);
	g->FillRect(0, 0, mWidth, mHeight);
}

void ObjectOrderTable::Update()
{
	Widget::Update();
}

TObject* ObjectOrderTable::Dispense()
{
	TObject* returnObj = NULL;
	
	switch(mState)
	{
	case WAITING_FILL:
	case WAITING_PICKUP:	if(mPizzaContainerSize > 0)
							{
								returnObj = mPizzaContainer[mPizzaContainerSize-1]->Copy(); //Dispense PIZZA
								returnObj->pSourceObject = this;
								
								mNumPizzaFilled--;
								
								for(int i=0; i<mOrder->mArraySize; i++)
								{
									if(mOrder->mArray[i] == returnObj->mType)
									{
										mPizzaFillArray[i] = false;
									}
								}
								
								removePizza(mPizzaContainer[mPizzaContainerSize-1]->mType);
							}
							break;
	}
	
	//return pFrame->mTObjectFactory->getTObject(TObjectFactory::PIZZA);
	return returnObj;
}

bool ObjectOrderTable::Consume(TObject* theObj)
{
	if(theObj!=NULL)
	{
		switch(mState)
		{
		case EMPTY:		if(theObj->mType == TObjectFactory::ORDER)
						{
							mOrder = theObj;
							theObj->pSourceObject->Delivered(theObj->mType);
							theObj->pSourceObject = this;

							initArrays();								

							setState(WAITING_FILL);

							pFrame->RemoveCursorObject(false);
							return false;
						}
						break;

		case WAITING_FILL:
		case WAITING_PICKUP:	for(int i=0; i<mOrder->mArraySize; i++)
								{
									
									if(theObj->mType == mOrder->mArray[i] && theObj->mCookedFood)
									{
										if(mPizzaDeliveryArray[i] == false && mPizzaFillArray[i] == false)
										{
											addPizza(theObj->Copy());

											mPizzaFillArray[i] = true;
											mNumPizzaFilled++;
											if(mNumPizzaFilled == mOrder->mArraySize)
												setState(WAITING_PICKUP);

											theObj->pSourceObject->Delivered(mType);
											return true;
										}
									}
								}
								break;
		}
	}

	return false;
}

void ObjectOrderTable::Delivered(short id)
{
	switch(mState)
	{
	case WAITING_FILL:
	case WAITING_PICKUP:	if(id != GameObjectFactory::ORDERTABLE)
							{
								DeliverPizza(id);
							}
							break;
	}
}	

void ObjectOrderTable::DeliverPizza(short id)
{
	for(int i=0; i<mOrder->mArraySize; i++)
	{
		if(id == mOrder->mArray[i])
		{
			mPizzaDeliveryArray[i]=true;
			
			mNumPizzaFilled++;
			if(mNumPizzaFilled == mOrder->mArraySize)
				setState(WAITING_PICKUP);

			mNumPizzaDelivered++; 
			if(mNumPizzaDelivered == mOrder->mArraySize)
				setState(EMPTY);
			
			break;
		}
	}
}

void ObjectOrderTable::resizePizzaContainer(int newSize)
{
	if(newSize < mPizzaContainerSize)
	{
		for(int i=newSize; i<mPizzaContainerSize; i++)
		{
			delete mPizzaContainer[i];
			mPizzaContainer[i] = NULL;
		}
		
		TObject** tempArray = new TObject*[newSize];
		
		for(int i=0; i<newSize; i++)
		{
			tempArray[i] = mPizzaContainer[i];
		}

		delete mPizzaContainer;
		mPizzaContainer = new TObject*[newSize];

		for(int i=0; i<newSize; i++)
		{
			mPizzaContainer[i] = tempArray[i] ;
		}

		mPizzaContainerSize = newSize;
	}
	else if(newSize > mPizzaContainerSize)
	{
		TObject** tempArray = new TObject*[mPizzaContainerSize];

		for(int i=0; i<mPizzaContainerSize; i++)
		{
			tempArray[i] = mPizzaContainer[i];
		}

		if(mPizzaContainer != NULL)
		{
			delete mPizzaContainer;
		}

		mPizzaContainer = new TObject*[newSize];

		for(int i=0; i<mPizzaContainerSize; i++)
		{
			mPizzaContainer[i] = tempArray[i] ;
		}

		for(int i=mPizzaContainerSize; i<newSize; i++)
		{
			mPizzaContainer[i] = NULL;
		}

		mPizzaContainerSize = newSize;
	}
}

void ObjectOrderTable::addPizza(TObject* thePizza)
{
	resizePizzaContainer(mPizzaContainerSize+1);

	mPizzaContainer[mPizzaContainerSize-1] = thePizza;
}

bool ObjectOrderTable::removePizza(short theId)
{
	int removeIndex = -1;
	
	for(int i=0; i<mPizzaContainerSize; i++)
	{
		if(mPizzaContainer[mPizzaContainerSize-1-i]->mType == theId)
		{
			removeIndex = mPizzaContainerSize-1-i;
		}
	}

	if(removeIndex != -1)
	{
		//delete mPizzaContainer[removeIndex];

		for(int i=removeIndex; i<mPizzaContainerSize-1; i++)
		{
			mPizzaContainer[i] = mPizzaContainer[i+1];
		}

		resizePizzaContainer(mPizzaContainerSize-1);

		return true;
	}
	else
	{
		return false;
	}
}

void ObjectOrderTable::initArrays()
{
	resizePizzaContainer(0);
	mPizzaContainerSize = 0;
		
	mPizzaArraySize = mOrder->mArraySize;

	mPizzaDeliveryArray = new bool[mPizzaArraySize];
	mPizzaFillArray = new bool[mPizzaArraySize];

	for(int i=0; i<mPizzaArraySize; i++)
	{
		mPizzaDeliveryArray[i] = false;
		mPizzaFillArray[i] = false;
	}

	mNumPizzaDelivered = 0;
	mNumPizzaFilled = 0;
}

void ObjectOrderTable::releaseArrays()
{	
	if(mPizzaContainer != NULL)
	{
		for(int i=0; i<mPizzaContainerSize; i++)
		{
			if(mPizzaContainer[i] != NULL)
				delete mPizzaContainer[i];
		}
		delete[] mPizzaContainer;
	}

	if(mPizzaFillArray != NULL)
	{
		delete mPizzaFillArray;
	}

	if(mPizzaDeliveryArray != NULL)
	{
		delete mPizzaDeliveryArray;
	}

	mPizzaContainerSize = 0;
	mPizzaArraySize = 0;

	mNumPizzaDelivered = 0;
	mNumPizzaFilled = 0;
}

void ObjectOrderTable::setState(eStateType newStatus)
{
	mState = newStatus;

	switch(mState)
	{
	case EMPTY:			mConsumer = true; 
						mDispenser = false; 
						
						releaseArrays();
						// Wait to consume orders
						break;

	case WAITING_FILL:	mConsumer = true; 
						mDispenser = true; 
						// Wait to fill orders
						// Pickup Enabled
						break;


	case WAITING_PICKUP:	mConsumer = true; 
							mDispenser = true; 
							// Wait for complete Pickup
							break;
	}

}
