#include "GameObjectFactory.h"
#include "GameObject.h"
#include "ObjectOven.h"
#include "GameApp.h"
#include "GameFrame.h"

#include "Tray.h"
#include "TObject.h"
#include "TObjectFactory.h"

#include "../../../SexyAppFramework/WidgetManager.h"

using namespace Sexy;

ObjectOven::ObjectOven(GameApp* theApp, GameFrame* theFrame):GameObject(theApp, theFrame)
{
	mType = GameObjectFactory::OVEN;
	mName = "Oven";
	pImg = pApp->mOvenImg;

	mActive = true;

	mCountdown = -1;

	mPizza = NULL;
	mDrawCountdownImg = false;

	setState(EMPTY);

	//Resize(0, 0, 48, 96);
}

ObjectOven::~ObjectOven()
{
	if(mPizza != NULL)
		delete mPizza;
}

void ObjectOven::Draw(Graphics* g)
{
	g->DrawImage(pImg,0,0,mWidth,mHeight);

	if(mDrawCountdownImg)
	{	
		g->SetColor(mPizza->mColor);
		g->FillRect(10, 10, 30, 30);

		g->SetColor(Color(0,0,0));
		g->DrawRect(10, 10, 29, 29);
	}
	
	if(mCountdown > 0)
	{
		short alpha255 = mCountdown*255/200;
		Color c;
		int sum;

		sum = alpha255 + mPizza->mColor.mRed;
		if(sum > 255) sum = 255;
		c.mRed = sum;

		sum = alpha255 + mPizza->mColor.mGreen;
		if(sum > 255) sum = 255;
		c.mGreen = sum;

		sum = alpha255 + mPizza->mColor.mBlue;
		if(sum > 255) sum = 255;
		c.mBlue = sum;

		g->SetColor(c);

		g->FillRect(10, 10, 30, 30);

		g->SetColor(Color(0,0,0));
		g->DrawRect(10, 10, 29, 29);
		
		g->mColorizeImages = true;
		short alpha = mCountdown*127/500;
		g->SetColor(Color(255-alpha,255-alpha,255-alpha));
	}
	else
	{
		g->mColorizeImages = false;
	}

	if(mDrawCountdownImg)
	{
		g->DrawImage(mPizza->pImg, 10, 10, 30, 30);
	}

	g->mColorizeImages = false;

	g->SetColor(mColor);
	g->FillRect(0, 0, mWidth, mHeight);
}

void ObjectOven::Update()
{
	Widget::Update();
	
	if(mCountdown == 0)
	{
		mPizza->mCookedFood = true;
		setState(WAITING_PICKUP);
		mCountdown--;

		pApp->PlaySample(GameApp::SFX_Ding);
	}
	else if(mCountdown > 0)
	{
		mCountdown--;
		pFrame->MarkDirty();
	}
}

TObject* ObjectOven::Dispense()
{
	TObject* returnObj = NULL;
	
	switch(mState)
	{
	case WAITING_PICKUP:	if(mPizza != NULL)
							{
								returnObj = mPizza->Copy();
								returnObj->pSourceObject = this;
								mDrawCountdownImg = false;
								delete mPizza;
								mPizza = NULL;
								setState(EMPTY); 
								pApp->PlaySample(GameApp::SFX_Click);
							}
							break;
	}

	return returnObj;
}

bool ObjectOven::Consume(Tray* theTray)
{
	
	if(theTray == NULL)
	{
		/*
		switch(mState)
		{
		case COOKING:			
								
		case WAITING_PICKUP:	setState(EMPTY);
								pApp->PlaySample(GameApp::SFX_TrashCan);
								break;
								
		}
		*/
		pApp->PlaySample(GameApp::SFX_Thud);
		return false;
	}
	

	if(!theTray->isEmpty())
	{
		switch(mState)
		{
		case EMPTY:				for(int i=0; i<theTray->getNumItemsInTray(); i++)
								{
									TObject* theObj = theTray->getItemByIndex(theTray->getNumItemsInTray()-1-i);
									if(!theObj->mIngredient && !theObj->mCookedFood)
									{
										mPizza = theObj->Copy();
										//theObj->pSourceObject->Delivered(mType);
										setState(COOKING);

										theTray->removeItem(theObj);
										delete theObj;

										pApp->PlaySample(GameApp::SFX_Click);
										return true;
									}
								}
								break;

		case WAITING_PICKUP:    TObject* topitem = theTray->getTopItem();
								if(topitem->pSourceObject == this && mPizza == NULL)
								{
									mPizza = topitem->Copy();
									mPizza->pSourceObject->Delivered(mType);
									mDrawCountdownImg = true;

									theTray->removeItem(topitem);
									delete topitem;

									pApp->PlaySample(GameApp::SFX_Click);
									return true;
								}
								break;
		}
		
	}

	return false;
}

void ObjectOven::Delivered(short id)
{
	switch(mState)
	{
	case WAITING_PICKUP:	switch(id)
							{
							case GameObjectFactory::TRASH:
							case GameObjectFactory::TABLE:
							case GameObjectFactory::ORDERTABLE:	setState(EMPTY); 
																break;
							}
							break;
	}
}	

void ObjectOven::setState(eStateType newStatus)
{
	mState = newStatus;

	switch(mState)
	{
	case EMPTY:			mConsumer = true; 
						mDispenser = false;

						mCountdown = -1;
						mDrawCountdownImg = false;

						if(mPizza != NULL)
						{
							delete mPizza;
							mPizza = NULL;
						}

						// Wait for uncooked pizza
						break;

	case COOKING:		mConsumer = false; 
						mDispenser = false; 
						mCountdown = 200;
						mDrawCountdownImg = true;
						// Wait for countdown
						break;


	case WAITING_PICKUP:	mConsumer = true; 
							mDispenser = true; 
							// Wait for Pickup
							break;
	}

	
	MarkDirty();
}

