#include "GameObjectFactory.h"
#include "GameObject.h"
#include "ObjectPrepTable.h"
#include "GameApp.h"
#include "GameFrame.h"
#include "Selector.h"

#include "Tray.h"
#include "TObject.h"
#include "TObjectFactory.h"

#include "../../../SexyAppFramework/WidgetManager.h"

using namespace Sexy;

ObjectPrepTable::ObjectPrepTable(GameApp* theApp, GameFrame* theFrame):GameObject(theApp, theFrame)
{
	mType = GameObjectFactory::PREPTABLE;
	mName = "Prep Table";
	pImg = pApp->mPrepTableImg;

	//Resize(0, 0, 50, 50);

	mIngredientContainer = NULL;
	mIngredientSpecs = NULL;
	mPizza = NULL;

	mPizzaSelector = NULL;
	mSelectedPizza = -1;

	mPizzaSelector = new Selector(pApp, pFrame, this);

	setState(EMPTY);
}

ObjectPrepTable::~ObjectPrepTable()
{
	if(mIngredientContainer != NULL)
		delete mIngredientContainer;

	if(mIngredientSpecs != NULL)
		delete mIngredientSpecs;

	if(mPizza != NULL)
		delete mPizza;

	if(mPizzaSelector != NULL)
		delete mPizzaSelector;
}

void ObjectPrepTable::Draw(Graphics* g)
{
	//g->SetColor(Color(0, 0, 127));
	//g->FillRect(0, 0, mWidth, mHeight);
	g->DrawImage(pImg,0,0,mWidth,mHeight);

	switch(mState)
	{
	case WAITING_FILL:		//#CANCEL# Display Pizza dough graphic (unfinished pizza)
							//Display Ingredients needed
							{
								Color c = mPizza->mColor;
								c.mAlpha = 127;
								g->SetColor(c);
								g->FillRect(0, 0, mWidth, mHeight);
								
								int x = 0;
								int y = 0;

								switch(mPizza->mArraySize)
								{
								case 1: x += 12; y += 12; break;
								case 2: x += 0; y += 12; break;
								case 3: 
								case 4: x += 0; y += 0; break;
								} 
						
								for(int i=0; i<mPizza->mArraySize; i++)
								{
									bool ingredienthere = false;
									
									for(int j=0; j<mIngredientContainerSize; j++)
									{
										if(mIngredientContainer[j])
										{
											if(mPizza->mArray[i] == mIngredientSpecs[j])
												ingredienthere = true;
				
										}
									}

									if(!ingredienthere)
									{
										g->SetColor(Color(255,255,255,127));
										g->FillRect(x, y, 25, 25);

										g->SetColor(Color(0,0,0));
										g->DrawRect(x, y, 24, 24);

										TObject* tempObj = pFrame->mTObjectFactory->getTObject(mPizza->mArray[i]);
										
										g->DrawImage(tempObj->pImg, x, y, 25, 25);

										delete tempObj;
									}

									g->mColorizeImages = false;

									switch(i)
									{
									case 0: x+=25; break;
									case 1: x-=25; y+=25; break;
									case 2: x+=25; break;
									}
								}
							}
							break;

	case WAITING_PICKUP:	//Display finished uncooked Pizza
							if(mShowPizza)
							{
								Color c = mPizza->mColor;
								c.mAlpha = 127;
								g->SetColor(c);
								g->FillRect(0, 0, mWidth, mHeight);
								
								g->SetColor(Color(255,255,255,127));
								g->FillRect(10, 10, 30, 30);

								g->SetColor(Color(0,0,0));
								g->DrawRect(10, 10, 29, 29);

								g->mColorizeImages = true;
								g->SetColor(Color(190,190,190));			
								g->DrawImage(mPizza->pImg, 10, 10, 30, 30);
								g->mColorizeImages = false;
							}
							break;
	}
	
	g->SetColor(mColor);
	g->FillRect(0, 0, mWidth, mHeight);
}

void ObjectPrepTable::Update()
{
	Widget::Update();
}

void ObjectPrepTable::AddedToManager(WidgetManager* theWidgetManager)
{
	Widget::AddedToManager(theWidgetManager);

	mWidgetManager->AddWidget(mPizzaSelector);

	mPizza = pFrame->mTObjectFactory->getTObject(TObjectFactory::MARGHERITA);
	mPizzaSelector->AddItem(mPizza->mType, mPizza->pImg, mPizza->mName, mPizza->mColor);
	delete mPizza;

	mPizza = pFrame->mTObjectFactory->getTObject(TObjectFactory::MARINARA);
	mPizzaSelector->AddItem(mPizza->mType, mPizza->pImg, mPizza->mName, mPizza->mColor);
	delete mPizza;

	mPizza = pFrame->mTObjectFactory->getTObject(TObjectFactory::CIPOLLA);
	mPizzaSelector->AddItem(mPizza->mType, mPizza->pImg, mPizza->mName, mPizza->mColor);
	delete mPizza;

	mPizza = pFrame->mTObjectFactory->getTObject(TObjectFactory::NAPOLETANA);
	mPizzaSelector->AddItem(mPizza->mType, mPizza->pImg, mPizza->mName, mPizza->mColor);
	delete mPizza;

	mPizza = pFrame->mTObjectFactory->getTObject(TObjectFactory::VIENNESE);
	mPizzaSelector->AddItem(mPizza->mType, mPizza->pImg, mPizza->mName, mPizza->mColor);
	delete mPizza;

	mPizza = NULL;
}

void ObjectPrepTable::RemovedFromManager(WidgetManager* theWidgetManager)
{
	mWidgetManager->RemoveWidget(mPizzaSelector);

	Widget::RemovedFromManager(theWidgetManager);
}

TObject* ObjectPrepTable::Dispense()
{
	TObject* returnObj = NULL;
	
	switch(mState)
	{
	case EMPTY:		if(mSelectedPizza != -1)
						setPizza(mSelectedPizza, true);
					returnObj = NULL;
					break;

	case WAITING_PICKUP:	if (mPizza != NULL)
							{
								returnObj = mPizza->Copy(); //Dispense PIZZA
								returnObj->pSourceObject = this;
								mShowPizza = false;
								delete mPizza;
								mPizza = NULL;
								setState(EMPTY); 
							}
							break;
	}

	if(returnObj != NULL)
	{
		pApp->PlaySample(GameApp::SFX_Click);
	}

	return returnObj;
}

bool ObjectPrepTable::Consume(Tray* theTray)
{
	if(theTray == NULL)
	{
		switch(mState)
		{
		case WAITING_FILL:		setState(EMPTY);
								mPizzaSelector->ShowSelector(true);
								pApp->PlaySample(GameApp::SFX_TrashCan);
								break;

		default:				pApp->PlaySample(GameApp::SFX_Thud);
								break;
		}


		return false;
	}
	
	switch(mState)
	{
	case EMPTY:				if(mSelectedPizza != -1)
								setPizza(mSelectedPizza, true);
							pApp->PlaySample(GameApp::SFX_Click);
							return true;
							break;
	}
	
	if(!theTray->isEmpty())
	{
		switch(mState)
		{
		case WAITING_FILL:		{
								bool consumed = false;
								for(int i=0; i<mIngredientContainerSize; i++)
								{
									if(theTray->getNumElementInTrayOfId(mIngredientSpecs[i]) > 0)
									{
										if(mIngredientContainer[i] == false)
										{
											TObject* theObj = theTray->getTopItemById(mIngredientSpecs[i]);
											theTray->removeItem(theObj);
											delete theObj;
								
											mIngredientContainer[i] = true;
											mNumIngredientContained++;
											if(mNumIngredientContained == mIngredientContainerSize)
												setState(WAITING_PICKUP);
											
											consumed = true;
											pApp->PlaySample(GameApp::SFX_Click);
											MarkDirty();
										}
									}
								}
								return consumed;
								}
								break;

		case WAITING_PICKUP:    TObject* topitem = theTray->getTopItem();
								if(topitem->pSourceObject == this && mPizza == NULL)
								{
									mPizza = topitem->Copy();
									//mPizza->pSourceObject->Delivered(mType);

									mShowPizza = true;

									theTray->removeItem(topitem);
									delete topitem;
									return true;
								}
								break;
		}
		
	}
	/*
	else
	{
		switch(mState)
		{
		case WAITING_FILL:		setState(EMPTY);
								mPizzaSelector->ShowSelector(true);
								break;
		}
	}
	*/

	return false;
}

void ObjectPrepTable::Delivered(short id)
{
	switch(mState)
	{
	case EMPTY:				mSelectedPizza = id;
							break;

	case WAITING_PICKUP:	switch(id)
							{
							case GameObjectFactory::OVEN:
							case GameObjectFactory::TRASH:  setState(EMPTY);
															break;
							}
							break;
	}
}	

void ObjectPrepTable::MouseEnter()
{
	if(mSelectable)
	{
		if(mState == EMPTY)
		{
			mPizzaSelector->ShowSelector(true);
		}

		mColor = mHighlightColor;
		pFrame->MarkDirty();
	}
}

void ObjectPrepTable::MouseLeave()
{
	if(mSelectable)
	{
		if(mState == EMPTY)
			mPizzaSelector->ShowSelector(false);

		mColor = mNormalColor;
		pFrame->MarkDirty();
	}
}

void ObjectPrepTable::setState(eStateType newStatus)
{
	mState = newStatus;

	switch(mState)
	{
	case EMPTY:			mConsumer = true; 
						mDispenser = true;

						mShowPizza = true;

						setPizza(0, false);
						//mPizzaSelector->ShowSelector(true);
						mSelectedPizza = -1;
						// Wait for user-select template
						break;

	case WAITING_FILL:	mConsumer = true; 
						mDispenser = false; 

						mPizzaSelector->ShowSelector(false);
						pFrame->MarkDirty();
						// Wait to fill ingredients
						// Pickup Disabled
						break;


	case WAITING_PICKUP:	mConsumer = true; 
							mDispenser = true; 
							// Wait for Pickup
							break;
	}

}

void ObjectPrepTable::setPizza(short id, bool flag)
{
	if(flag == true && mPizza == NULL)
	{
		mPizza = pFrame->mTObjectFactory->getTObject((TObjectFactory::eObjectName)id);
		mIngredientContainerSize = mPizza->mArraySize;
		mNumIngredientContained = 0;
		mIngredientSpecs = new short[mIngredientContainerSize];
		mIngredientContainer = new bool[mIngredientContainerSize];	
		for(int i=0; i<mIngredientContainerSize; i++)
		{
			mIngredientSpecs[i] = mPizza->mArray[i];
			mIngredientContainer[i] = false;
		}

		setState(WAITING_FILL);
	}
	else
	{
		if(mIngredientSpecs != NULL) {
			delete mIngredientSpecs;
			mIngredientSpecs = NULL;
		}

		if(mIngredientContainer != NULL) {
			delete mIngredientContainer;
			mIngredientContainer = NULL;
		}

		if(mPizza != NULL) {
			delete mPizza;
			mPizza = NULL;
		}
	}
}
