#include "GameObjectFactory.h"
#include "GameObject.h"
#include "ObjectTable.h"
#include "ObjectChair.h"

#include "PopupMsg.h"

#include "GameApp.h"
#include "GameFrame.h"

#include "GameSpace.h"
#include "GameNode.h"

#include "Tray.h"
#include "TObject.h"
#include "TObjectFactory.h"

#include "Customer.h"

#include "../../../SexyAppFramework/WidgetManager.h"

using namespace Sexy;

ObjectTable::ObjectTable(GameApp* theApp, GameFrame* theFrame):GameObject(theApp, theFrame)
{
	mType = GameObjectFactory::TABLE;
	mName = "Table";
	pImg = pApp->mTableImg;

	//Resize(0, 0, 50, 50);

	mNumCustomers = 0;

	mNumChairs = 4;
	mChairArraySize = 4;
	mChairArray = new ObjectChair*[mChairArraySize];
	mMenuArray = new bool[mChairArraySize];
	mPlateArray = new bool[mChairArraySize];
	mDirtyArray = new bool[mChairArraySize];

	//pPizzaImgArray = new Image*[mChairArraySize];
	//mPizzaDeliveredArray = new short[mChairArraySize];

	mPizzaArray = new TObject*[mChairArraySize];
	
	for(int i=0; i<mChairArraySize; i++)
	{
		mChairArray[i] = NULL;
		mMenuArray[i] = false;
		mPlateArray[i] = false;
		mDirtyArray[i] = false;

		//mPizzaDeliveredArray[i] = -1;

		mPizzaArray[i] = NULL;
	}

	mTimer = -1;
	mCountdown = -1;

	mTotalWaitTime = 0;

	mPizzaDelivered = 0;

	mOrder = NULL;

	setState(EMPTY);
}

ObjectTable::~ObjectTable()
{
	for(int i=0; i<mChairArraySize; i++)
	{
		delete mChairArray[i];

		if(mPizzaArray[i] != NULL)
		{
			delete mPizzaArray[i];
		}
	}
	delete[] mChairArray;

	delete mMenuArray;
	delete mPlateArray;
	delete mDirtyArray;

	//delete pPizzaImgArray;
	//delete mPizzaDeliveredArray;

	delete mPizzaArray;

	if(mOrder != NULL)
	{
		delete mOrder;
	}
}

void ObjectTable::AddedToManager(WidgetManager* theWidgetManager)
{
	Widget::AddedToManager(theWidgetManager);
}

void ObjectTable::RemovedFromManager(WidgetManager* theWidgetManager)
{
	for(int i=0; i<mChairArraySize; i++)
	{
		if(mChairArray[i] != NULL)
		{
			mWidgetManager->RemoveWidget(mChairArray[i]);
		}
	}

	Widget::RemovedFromManager(theWidgetManager);
}

void ObjectTable::Draw(Graphics* g)
{
	//g->SetColor(Color(127, 0, 127));
	//g->FillRect(0, 0, 50, 50);
	g->DrawImage(pImg,0,0,mWidth,mHeight);
	
	
	for(int i=0; i<mChairArraySize; i++)
	{
		g->SetColor(Color(171, 171, 171));
		if(mPlateArray[i] == true)
		{
			switch(i)
			{
			case 0: g->FillRect(16, 0, 18, 18); break;
			case 1: g->FillRect(32, 16, 18, 18); break;
			case 2: g->FillRect(16, 32, 18, 18); break;
			case 3: g->FillRect(0, 16, 18, 18); break;
			}
		}
		/*
		g->SetColor(Color(0, 171, 171));
		if(mDirtyArray[i] == true)
		{
			switch(i)
			{
			case 0: g->FillRect(16, 0, 18, 18); break;
			case 1: g->FillRect(32, 16, 18, 18); break;
			case 2: g->FillRect(16, 32, 18, 18); break;
			case 3: g->FillRect(0, 16, 18, 18); break;
			}
		}

		g->SetColor(Color(171, 0, 171));
		if(mMenuArray[i] == true)
		{
			switch(i)
			{
			case 0: g->FillRect(16, 0, 18, 18); break;
			case 1: g->FillRect(32, 16, 18, 18); break;
			case 2: g->FillRect(16, 32, 18, 18); break;
			case 3: g->FillRect(0, 16, 18, 18); break;
			}
		}
		*/
	}

	Color c = Color(255,0,0);
	c.mAlpha = ((mTimer + mTotalWaitTime)/6000.0f) * 159;
	g->SetColor(c);
	g->FillRect(0, 0, 50, 50);


	switch(mState)
	{
	case WAITING_FOOD:
	case EATING:	{				
					int x=0;
					int y=0;

					for(int i=0; i<mNumCustomers; i++)
					{
						if(mPizzaArray[i]->mDelivered == true)
						{
							switch(i)
							{
							case 0: x=16; y=0; break;
							case 1: x=32; y=16; break;
							case 2: x=16; y=32; break;
							case 3: x=0; y=16; break;
							}
						
							g->DrawImage(mPizzaArray[i]->pImg, x-1, y-1, 20, 20);
						}
					}
				}
				break;
				

	case DIRTY:
					g->SetColor(Color(0,0,0,127));
					g->FillRect(0, 0, 50, 50);
					g->mColorizeImages = true;
					g->SetColor(Color(255,255,0));
					g->DrawImage(pApp->mEuroImg, 5,5);
					g->mColorizeImages = true;
					break;
	}	

	g->SetColor(mColor);
	g->FillRect(0, 0, 50, 50);
}

void ObjectTable::Update()
{
	Widget::Update();

	if(!pFrame->mPaused && !pFrame->mGamePaused)
	{

		if(mCountdown == 0)
		{
			switch(mState)
			{
			case EMPTY:

								break;

			case DECIDING:		// Random Generate ORDERs
								generateOrder();
								// Call for Order (Sound and Graphics)
								setState(WAITING_ORDER);
								setState(WAITING_FOOD);
								break;

			case EATING:		// Call for bill (Sound and Graphics)
								setState(WAITING_BILL);
								break;
			}

			mCountdown--;
		}
		else if(mCountdown > 0)
		{
			mCountdown--;
		}

		if(mTimer >= 0)
		{
			mTimer++;
		}

		if(mState != DIRTY && mTimer + mTotalWaitTime == 4500)
		{
			PopupMsg* popup = new PopupMsg(pApp, pFrame, 250);
			popup->Center(mX+25, mY+5);
			popup->mStringColor1= Color(63,63,63);
			popup->mDisplayString1 = "Hurry Up!";
			mWidgetManager->AddWidget(popup);
			
			pApp->PlaySample(GameApp::SFX_HurryUp);
		}

		if(mState != DIRTY && mTimer + mTotalWaitTime > 6000)
		{
			PopupMsg* popup = new PopupMsg(pApp, pFrame, 250);
			popup->Center(mX+25, mY+5);

			int addRep = -10*(mNumCustomers-mPizzaDelivered)*(pFrame->mLevel+1/3.0f);
			pFrame->mReputation += addRep;
			popup->mStringColor1= Color(160,0,0);
			popup->mImageColor1= Color(190,127,127);
			popup->mDisplayString1 = StrFormat("- %d", -addRep);
			popup->pDisplayImage1 = pApp->mHeartImg;
			mWidgetManager->AddWidget(popup);

			pApp->PlaySample(GameApp::SFX_DoorSlam);
			
			setState(EMPTY);
		}
	}

}

TObject* ObjectTable::Dispense()
{
	TObject* returnObj = NULL;
	
	switch(mState)
	{
	case WAITING_MENU:	//mTotalWaitTime += mTimer;
						//mTimer = -1;
						setState(DECIDING);
						returnObj = NULL;
						break;
	
	case WAITING_ORDER: //Remove Menu
						//returnObj = mOrder; //Dispense ORDER, add delay
						//returnObj->pSourceObject = this;
						setState(WAITING_FOOD);
						break;

	case WAITING_BILL:	//mTotalWaitTime += mTimer;
						//mTimer = -1;
						// calculate earnings and tips 
						// call GameFrame to add CASH
						setState(DIRTY);
						returnObj = NULL;
						break;

	case DIRTY:			
						returnObj = NULL; //Dispense DIRTYDISHES
						break;

	}
	
	pFrame->MarkDirty();

	if(returnObj != NULL)
	{
		pApp->PlaySample(GameApp::SFX_Click);
	}

	return returnObj;
}

bool ObjectTable::Consume(Tray* theTray)
{
	if(theTray == NULL)
	{
		pApp->PlaySample(GameApp::SFX_Thud);
		return false;
	}
	
	switch(mState)
	{
	case WAITING_MENU: 
						//mTotalWaitTime += mTimer;
						//mTimer = -1;
						setState(DECIDING);
						break;
	case WAITING_ORDER: 
						setState(WAITING_FOOD);
						break;
	case WAITING_BILL:	
						//mTotalWaitTime += mTimer;
						//mTimer = -1;
						// calculate earnings and tips 
						// call GameFrame to add CASH
						setState(DIRTY);
						break;

	case DIRTY:			{
							mTotalWaitTime += mTimer;
							mTimer = -1;

							//pApp->Popup(StrFormat("%d", mTotalWaitTime));
		
							PopupMsg* popup = new PopupMsg(pApp, pFrame, 250);
							popup->Center(mX+25, mY+5);
		
							int waitTimePerPerson = mTotalWaitTime/400.0f;
							int tips = (15.0f-waitTimePerPerson)/3.0f;
							if(tips < 0) tips = 0;
							int fundsreward = 0;
							for(int i=0; i<mOrder->mArraySize; i++)
							{
								switch(mOrder->mArray[i])
								{
								case TObjectFactory::MARGHERITA:
								case TObjectFactory::CIPOLLA:	fundsreward+=3+2*tips; break;

								case TObjectFactory::NAPOLETANA: 
								case TObjectFactory::VIENNESE:	fundsreward+=6+4*tips; break;

								case TObjectFactory::MARINARA:	fundsreward+=9+6*tips; break;
								}
							}

							fundsreward = fundsreward*(pFrame->mLevel+1/3.0f);

							if (fundsreward > 0)
							{
							pFrame->mFunds += (pFrame->mLevel+1)*fundsreward;
							popup->mStringColor2= Color(0,0,160);
							popup->mDisplayString2 = StrFormat("+ %d", fundsreward);
							popup->pDisplayImage2 = pApp->mEuroImg;
							}
						
							int repreward = (15.0f-waitTimePerPerson)/3.0f *(pFrame->mLevel+1/2.0f);
							if (repreward == 0) repreward = 1;
							pFrame->mReputation += repreward;
							if(repreward < 0)
							{
								popup->mStringColor1= Color(160,0,0);
								popup->mImageColor1= Color(190,127,127);
								popup->mDisplayString1 = StrFormat("- %d", repreward);
							}
							else
							{
								popup->mStringColor1= Color(0,160,0);
								popup->mImageColor1= Color(255,255,255);
								popup->mDisplayString1 = StrFormat("+ %d", repreward);
							}
							popup->pDisplayImage1 = pApp->mHeartImg;
							mWidgetManager->AddWidget(popup);

							setState(EMPTY);
							pApp->PlaySample(GameApp::SFX_Click);
							pApp->PlaySample(GameApp::SFX_CashRegister);
							return false;
						}
						break;
	}
	
	if(!theTray->isEmpty())
	{

		switch(mState)
		{
		case WAITING_FOOD:	{
								bool consumed = false;
								int delivered = 0;
								int bonus = 0;

								for(int i=0; i<mOrder->mArraySize; i++)
								{
									if(theTray->getNumElementInTrayOfId(mOrder->mArray[i]) > 0)
									{	
										TObject* theObj = theTray->getTopItemById(mOrder->mArray[i]);
										if(theObj->mCookedFood)
										{
											if(mPizzaArray[i]->mDelivered != true)
											{
												//theObj->pSourceObject->Delivered(mType);

												theTray->removeItem(theObj);
												delete theObj;
												
												mPizzaArray[i]->mDelivered = true;
												mChairArray[i]->DisplayImage(NULL, Color(0, 0, 0, 0));
												mPlateArray[i] = true;

												mPizzaDelivered++;
												delivered++;

												switch(mPizzaArray[i]->mType)
												{
												case TObjectFactory::MARGHERITA:
												case TObjectFactory::CIPOLLA:	bonus+=1; break;

												case TObjectFactory::NAPOLETANA: 
												case TObjectFactory::VIENNESE:	bonus+=2; break;

												case TObjectFactory::MARINARA:	bonus+=3; break;
												}
											
												pFrame->MarkDirty();

												consumed = true;

												if(mPizzaDelivered == mOrder->mArraySize)
												{
													break;
												}
											}
										}
										else
										{
											PopupMsg* popup = new PopupMsg(pApp, pFrame, 250);
											popup->Center(mChairArray[i]->mX+25, mChairArray[i]->mY+5);
											popup->mStringColor1 = Color(160,0,0);
											popup->mDisplayString1 = "Not Cooked!";
											int penalty = pFrame->mLevel+1/2.0f;
											if (penalty == 0) penalty = 1;
											pFrame->mReputation += -penalty;
										
											popup->mStringColor2= Color(160,0,0);
											popup->mImageColor2= Color(190,127,127);
											popup->mDisplayString2 = StrFormat("- %d", penalty);
											popup->pDisplayImage2 = pApp->mHeartImg;
											mWidgetManager->AddWidget(popup);

											pApp->PlaySample(GameApp::SFX_Thud);
										}
									} 
								}

								if(delivered > 0)
								{
									PopupMsg* popup = new PopupMsg(pApp, pFrame, 250);
									popup->Center(mX+25, mY+5);

									int waitTime = mTimer;
									int repreward = (3000.0f-waitTime)/100.0f/5.0f*delivered*(pFrame->mLevel+1/2.0f);
									if (repreward == 0) repreward = 1;

									popup->mStringColor1= Color(0,160,0);
									popup->mImageColor1= Color(255,255,255);
									std::string repString = StrFormat("+ %d", repreward);
									if(repreward < 0)
									{
										popup->mStringColor1= Color(160,0,0);
										popup->mImageColor1= Color(190,127,127);
										repString = StrFormat("- %d", -repreward);
									}
									popup->mDisplayString1 = repString;

									if(bonus > 0 && delivered > 1)
									{
										popup->mStringColor2= Color(0,160,0);
										popup->mImageColor2= Color(255,255,255);
										popup->pDisplayImage2 = pApp->mHeartImg;
										bonus = bonus*delivered/2*(pFrame->mLevel+1/2.0f);
										popup->mDisplayString2 = StrFormat("+ %d", bonus);

										pApp->PlaySample(GameApp::SFX_Clap);
										
									}
									else
									{
										bonus = 0;
									}
									popup->pDisplayImage1 = pApp->mHeartImg;
									mWidgetManager->AddWidget(popup);

									pFrame->mReputation += repreward+bonus;

									pApp->PlaySample(GameApp::SFX_Click);
								}

								if(mPizzaDelivered == mOrder->mArraySize)
								{
									setState(EATING);
								}

								return consumed;
							}
							break;
		}	
	}

	pFrame->MarkDirty();
	return false;
}

void ObjectTable::Spawn(int i)
{
	setChair(i, true);
}

void ObjectTable::Delivered(short id)
{
	switch(mState)
	{
	case WAITING_ORDER:		
							break;
	}
}	

GameSimpleNode ObjectTable::getClosestFNode(GameSimpleNode theNode)
{
	GameSimpleNode* node;
	int numNodes = 0;

	if(mFNode == mNode)
	{
		GameSimpleNode tempNode[4];
		
		tempNode[0] = GameSimpleNode(mNode.x-1, mNode.y-1);
		tempNode[1] = GameSimpleNode(mNode.x+1, mNode.y-1);
		tempNode[2] = GameSimpleNode(mNode.x+1, mNode.y+1);
		tempNode[3] = GameSimpleNode(mNode.x-1, mNode.y+1);
		
		for(int i=0; i<4; i++)
		{
			if(		tempNode[i].x >= 0 && tempNode[i].x < pFrame->mColMapSizeX
				&&	tempNode[i].y >= 0 && tempNode[i].y < pFrame->mColMapSizeY )
			{
				numNodes++;
			}
			else
			{
				tempNode[i] = mNode;
			}
		}

		node = new GameSimpleNode[numNodes];
		int f=0;

		for(int i=0; i<4; i++)
		{
			if(!(tempNode[i] == mNode))
			{
				node[f++] = tempNode[i];
			}
		}
	}
	else
	{
		numNodes = 1;
		node = new GameSimpleNode[numNodes];

		node[0] = mFNode;
	}

	int dx;
	int dy;

	int* dist = new int[numNodes];

	for(int i=0; i<numNodes; i++)
	{
		if(pFrame->mColMap[node[i].x][node[i].y] != 0)
		{
			dist[i] = 100;

		}
		else
		{
			dx = node[i].x - theNode.x;
			dy = node[i].y - theNode.y;

			if(dx < 0) dx *= -1;
			if(dy < 0) dy *= -1;

			dist[i] = dx + dy;
		}
	}

	int minIndex = -1;
	int min = 100;
		
	for(int i=0; i<numNodes; i++)
	{
		if(dist[i] < min)
		{
			min = dist[i];
			minIndex = i;
		}
	}

	GameSimpleNode returnNode = mNode;
		
	if(minIndex >= 0)
	{
		returnNode = GameSimpleNode(node[minIndex].x, node[minIndex].y);
	}

	delete[] node;
	delete[] dist;
	
	return returnNode;
}

void ObjectTable::setChair(int i, bool theFlag)
{
	if(i>=0 && i<mChairArraySize)
	{
		GameSimpleNode newNode;
		switch(i)
		{
		case 0: newNode = GameSimpleNode(mNode.x, mNode.y-1); break;
		case 1: newNode = GameSimpleNode(mNode.x+1, mNode.y); break;
		case 2: newNode = GameSimpleNode(mNode.x, mNode.y+1); break;
		case 3: newNode = GameSimpleNode(mNode.x-1, mNode.y); break;
		}

		if(theFlag == false)
		{
			if(mChairArray[i] != NULL)
			{
				mWidgetManager->RemoveWidget(mChairArray[i]);
				pFrame->mColMap[newNode.x][newNode.y] = 0;
				delete mChairArray[i];
				mChairArray[i] = NULL;

				mNumChairs--;
			}
		}
		else
		{
			if(mChairArray[i] == NULL)
			{
				mChairArray[i] = new ObjectChair(pApp, pFrame);

				mChairArray[i]->Resize(0, 0, pFrame->mGridSize, pFrame->mGridSize);
				GameSimpleNode globalCoord = pFrame->convertMapNodeToGlobalCoord(newNode);
				mChairArray[i]->Move(globalCoord.x, globalCoord.y);
				
				mChairArray[i]->Assign(newNode, mNode);
				pFrame->mColMap[newNode.x][newNode.y] = 1;
				mWidgetManager->AddWidget(mChairArray[i]);

				mChairArray[i]->pImg = pApp->mChairImg[i];

				mNumChairs++;
			}
		}
	}
}

void ObjectTable::setState(eStateType newStatus)
{
	mState = newStatus;

	switch(mState)
	{
	case EMPTY:			mConsumer = false; 
						mDispenser = false; 
						mTotalWaitTime = 0;
						mTimer = -1;
						mPizzaDelivered = 0;
						removeCustomers();
						for(int i=0; i<mChairArraySize; i++)
						{
							mMenuArray[i] = false;
							mPlateArray[i] = false;
							mDirtyArray[i] = false;

							if (mPizzaArray[i] != NULL)
							{
								delete mPizzaArray[i];
								mPizzaArray[i] = NULL;
							}
						}

						mNumChairs = 4;
						mNumCustomers = 0;
						// Wait for GameFrame to call Spawn()
						// mCountdown = 500;
						break;

	case WAITING_MENU:	mConsumer = true; 
						mDispenser = true; 
						mTimer = -1;
						setState(DECIDING);
						break;

	case DECIDING:		mConsumer = false; 
						mDispenser = false; 
						for(int i=0; i<mChairArraySize; i++)
						{
							if(mChairArray[i] != NULL)
							{	
								mMenuArray[i] = true;
							}
						}
						mCountdown = 200 + Rand()%101;
						mTimer = -1;
						// Set Random wait period
						break;

	case WAITING_ORDER: mConsumer = true; 
						mDispenser = true; 
						mTimer = -1;
						for(int i=0; i<mChairArraySize; i++)
						{
							mMenuArray[i] = false;
						}
						break;

	case WAITING_FOOD:	mConsumer = true; 
						mDispenser = false; 
						for(int i=0; i<mOrder->mArraySize; i++)
						{
							if(mChairArray[i]->pCustomer != NULL)
							{	
								mChairArray[i]->DisplayImage(mPizzaArray[i]->pImg, mPizzaArray[i]->mColor);
							}
						}
						mTimer = 0;
						break;

	case EATING:		mConsumer = false; 
						mDispenser = false; 
						for(int i=0; i<mChairArraySize; i++)
						{
							if(mChairArray[i]->pCustomer != NULL)
							{	
								mPlateArray[i] = true;
								mChairArray[i]->DisplayImage(NULL, Color(0, 0, 0,0));
							}
						}
						// Set Random eating period
						mTotalWaitTime += mTimer;
						mTimer = -1;
						mCountdown = 900 + Rand()%201;
						break;

	case WAITING_BILL:	mConsumer = true; 
						mDispenser = true; 
						mTimer = -1;
						for(int i=0; i<mChairArraySize; i++)
						{
							if(mPlateArray[i] == true)
							{	
								mPlateArray[i] = false;
								mDirtyArray[i] = true;
							}
						}
						setState(DIRTY);
						break;

	case DIRTY:			mConsumer = true; 
						mDispenser = false; 

						PopupMsg* popup = new PopupMsg(pApp, pFrame, 250);
						popup->Center(mX+25, mY+5);
						popup->mStringColor1= Color(63,63,63);
						popup->mDisplayString1 = "Check Please!";
						mWidgetManager->AddWidget(popup);

						pApp->PlaySample(GameApp::SFX_WhistleHere);

						//setState(EMPTY);
						break;
	}

	pFrame->MarkDirty();
}

void ObjectTable::generateOrder()
{
	if(mOrder != NULL)
	{
		delete mOrder;		
	}
	
	mOrder=pFrame->mTObjectFactory->getTObject(TObjectFactory::ORDER);

	mOrder->InitArray(mNumCustomers);

	for(int i=0; i<mNumCustomers; i++)
	{
		PopupMsg* popup = new PopupMsg(pApp, pFrame, 200);
		popup->Center(mChairArray[i]->mX+25, mChairArray[i]->mY+5);
		popup->mStringColor1= Color(0,0,0);
		mWidgetManager->AddWidget(popup);
		
		switch(mChairArray[i]->pCustomer->SelectPizza())
		{
		case 0:
			mOrder->mArray[i] = TObjectFactory::MARGHERITA;
			popup->mDisplayString1 = "Margherita";
			break;

		case 1:
			mOrder->mArray[i] = TObjectFactory::MARINARA;
			popup->mDisplayString1 = "Marinara";
			break;

		case 2:
			mOrder->mArray[i] = TObjectFactory::CIPOLLA;
			popup->mDisplayString1 = "Cipolla";
			break;

		case 3:
			mOrder->mArray[i] = TObjectFactory::NAPOLETANA;
			popup->mDisplayString1 = "Napoletana";
			break;

		case 4:
			mOrder->mArray[i] = TObjectFactory::VIENNESE;
			popup->mDisplayString1 = "Viennese";
			break;
		}

		TObject* tempObj = pFrame->mTObjectFactory->getTObject(mOrder->mArray[i]);
		mPizzaArray[i] = tempObj;
		Color c = tempObj->mColor;
		c.mRed -= 95;
		if (c.mRed < 0) c.mRed = 0;
		c.mGreen -= 95;
		if (c.mGreen < 0) c.mGreen = 0;
		c.mBlue -= 95;
		if (c.mBlue < 0) c.mBlue = 0;
		popup->mStringColor1 = c;
		//popup->pDisplayImage1 = tempObj->pImg;
		//delete tempObj;

	}
}

void ObjectTable::addCustomer(Customer* newCustomer)
{
	for(int j=0; j<mChairArraySize; j++)
	{
		if(mChairArray[j]->pCustomer == NULL)
		{
			mChairArray[j]->pCustomer = newCustomer;

			//assign images for customers according to direction and type
			mChairArray[j]->pCustomer->pImg = mChairArray[j]->pCustomer->pIdleImgArray[j];
			mChairArray[j]->pCustomer->Resize(0,0,pFrame->mGridSize,pFrame->mGridSize);

			//add them to Chair's Widget
			mChairArray[j]->AddWidget(newCustomer);

			break;
		}
	}

	if(mState == EMPTY)
		setState(ObjectTable::WAITING_MENU);

}

void ObjectTable::removeCustomers()
{
	for(int j=0; j<mChairArraySize; j++)
	{
		if(mChairArray[j] != NULL)
		{
			//remove customers from widget manager
			mChairArray[j]->RemoveAllWidgets();
			//remove customer objects through GameFrame
			pFrame->removeCustomer(mChairArray[j]->pCustomer);
			
			mChairArray[j]->pCustomer = NULL;
		}
	}
}

