#include "GameObject.h"
#include "GameApp.h"
#include "GameFrame.h"

#include "TObject.h"
#include "TObjectFactory.h"

#include "Selector.h"
#include "SelectorItem.h"

#include "../../../SexyAppFramework/ImageFont.h"

#include "../../../SexyAppFramework/WidgetManager.h"

using namespace Sexy;

Selector::Selector(GameApp* theApp, GameFrame* theFrame, GameObject* theSrcObj)
{
	pApp = theApp;
	pFrame = theFrame;
	pSourceObject = theSrcObj;

	mDisplayText = pSourceObject->mName;
	pImg = NULL;

	mSelectable = true;
	mDraw = false;
	mHighlightColor = Color(255, 255, 255, 63);
	mNormalColor = Color(0, 0, 0, 0);
	mColor = mNormalColor;

	mItemArraySize = 0;
	mItemArray = NULL;

	Resize(0, 0, 90, 50);

}

Selector::~Selector()
{
	if(mItemArray != NULL)
	{
		for(int i=0; i<mItemArraySize; i++)
		{
			delete mItemArray[i];
		}
		mItemArraySize = 0;
		delete[] mItemArray;
	}
}

void Selector::Draw(Graphics* g)
{
	if(mDraw)
	{
		g->SetColor(Color(0,0,0,190));
		g->FillRect(0, 25, mWidth, mHeight-25);

		g->SetColor(Color(255,255,255));
		g->SetFont(pApp->mSysFont);
		g->WriteString(mDisplayText, 0, 26+pApp->mSysFont->GetHeight()-3, mWidth, 0);
		
		g->SetColor(mColor);
		g->FillRect(0, 25, mWidth, mHeight-25);
	}
}

void Selector::Update()
{
	Widget::Update();
}

void Selector::AddedToManager(WidgetManager* theWidgetManager)
{
	Widget::AddedToManager(theWidgetManager);
}

void Selector::RemovedFromManager(WidgetManager* theWidgetManager)
{
	
	for(int i=0; i<mItemArraySize; i++)
	{
		mWidgetManager->RemoveWidget(mItemArray[i]);
	}

	Widget::RemovedFromManager(theWidgetManager);

}

void Selector::MouseMove(int x, int y)
{
	pFrame->MouseMove(x+mX,y+mY);
}

void Selector::MouseEnter()
{
	if(mSelectable)
	{
		mDraw = true;
		for(int i=0; i<mItemArraySize; i++)
		{
			mItemArray[i]->mDraw = true;
		}
	}

	pSourceObject->MouseEnter();
}

void Selector::MouseLeave()
{
	if(mSelectable)
	{
		mDraw = false;
		for(int i=0; i<mItemArraySize; i++)
		{
			mItemArray[i]->mDraw = false;
		}
	}

	mDisplayText = pSourceObject->mName;

	pSourceObject->MouseLeave();
}

void Selector::MouseDown(int x, int y, int theBtnNum, int theClickCount)
{
}

void Selector::MouseUp(int x, int y, int theBtnNum, int theClickCount)
{
	pSourceObject->MouseUp(x,y, theBtnNum, theClickCount);	
}

void Selector::AddItem(short theId, Image* theImg, std::string theText, Color theColor)
{
	SelectorItem** tempArray = new SelectorItem*[mItemArraySize];
	for(int i=0; i<mItemArraySize; i++)
	{
		tempArray[i] = mItemArray[i];
	}

	delete mItemArray;

	mItemArraySize++;
	mItemArray = new SelectorItem*[mItemArraySize];

	for(int i=0; i<mItemArraySize-1; i++)
	{
		mItemArray[i] = tempArray[i];
	}

	delete tempArray;

	SelectorItem* tempItem = new SelectorItem(pApp, pFrame, this, theColor);
	tempItem->mId = theId;
	tempItem->pImg = theImg;
	tempItem->mText = theText;

	mWidgetManager->AddWidget(tempItem);

	mItemArray[mItemArraySize-1] = tempItem;

	Reposition();
}

void Selector::RemoveItem(short theId)
{
	int removeIndex = -1;
	
	for(int i=0; i<mItemArraySize; i++)
	{
		if(mItemArray[mItemArraySize-1-i]->mId == theId)
		{
			removeIndex = mItemArraySize-1-i;
		}
	}

	if(removeIndex != -1)
	{
		mWidgetManager->RemoveWidget(mItemArray[removeIndex]);
		delete mItemArray[removeIndex];

		for(int i=removeIndex; i<mItemArraySize-1; i++)
		{
			mItemArray[i] = mItemArray[i+1];
		}

		mItemArraySize--;

		SelectorItem** tempArray = new SelectorItem*[mItemArraySize];
		for(int i=0; i<mItemArraySize; i++)
		{
			tempArray[i] = mItemArray[i];
		}

		delete mItemArray;

		mItemArray = new SelectorItem*[mItemArraySize];

		for(int i=0; i<mItemArraySize-1; i++)
		{
			mItemArray[i] = tempArray[i];
		}

		delete tempArray;
	}
	
	Reposition();
}

void Selector::Reposition()
{
	int newWidth = mItemArraySize*30;
	int newHeight = mHeight;

	if(newWidth < 75)
	{
		newWidth = 75;
	}

	int newX = pSourceObject->mX + (pSourceObject->mWidth)/2 - newWidth/2;
	int newY = pSourceObject->mY + (pSourceObject->mHeight)/2 - newHeight/2;
	
	Resize(newX, newY, newWidth, newHeight);

	for(int i=0; i<mItemArraySize; i++)
	{
		int newItemX = newX + (newWidth/2 - mItemArraySize*30/2 ) + i*mItemArray[i]->mWidth;
		int newItemY = newY;

		mItemArray[i]->Move(newItemX, newItemY);
	}
}

void Selector::ShowSelector(bool flag)
{
	//pFrame->mWidgetManager->BringToFront(this);
	
	for(int i=0; i<mItemArraySize; i++)
	{
		mItemArray[i]->mVisible = flag;
		//pFrame->mWidgetManager->BringToFront(mItemArray[i]);
	}

	if(mVisible==false && flag==true)
	{
		mVisible = flag;
		MouseEnter();
	}
	else if(mVisible==true && flag==false)
	{
		mVisible = flag;
		//MouseLeave();
	}
}

