#include "GameObject.h"
#include "GameApp.h"
#include "GameFrame.h"

#include "TObject.h"
#include "TObjectFactory.h"

#include "Selector.h"
#include "SelectorItem.h"

#include "../../../SexyAppFramework/WidgetManager.h"

using namespace Sexy;

SelectorItem::SelectorItem(GameApp* theApp, GameFrame* theFrame, Selector* theSelector, Color theBgColor)
{
	pApp = theApp;
	pFrame = theFrame;
	pSelector = theSelector;

	mText = "";
	pImg = NULL;
	mId = -1;

	mSelectable = true;
	mDraw = false;
	mBgColor = theBgColor;
	mHighlightColor = Color(255, 255, 255, 63);
	mNormalColor = Color(0, 0, 0, 0);
	mColor = mNormalColor;

	Resize(0, 0, 30, 30);
}

SelectorItem::~SelectorItem()
{

}

void SelectorItem::Draw(Graphics* g)
{
	if(mDraw)
	{
		g->SetColor(Color(mBgColor.mRed, mBgColor.mGreen, mBgColor.mBlue, 190));

		g->FillRect(0, 0, mWidth, mHeight);

		g->SetColor(Color(0,0,0));
		g->DrawRect(0, 0, mWidth-1, mHeight-1);
		
		if(pImg == NULL)
		{
			g->SetColor(Color(0,0,255));
			g->FillRect(0, 0, mWidth, mHeight);
		}
		else
		{
			g->DrawImage(pImg, 0, 0, mWidth, mHeight);
		}
		
		g->SetColor(mColor);
		g->FillRect(0, 0, mWidth, mHeight);
	}
}

void SelectorItem::Update()
{
	Widget::Update();
}

void SelectorItem::MouseMove(int x, int y)
{
	pFrame->MouseMove(x+mX,y+mY);
}

void SelectorItem::MouseEnter()
{
	if(mSelectable)
	{
		mColor = mHighlightColor;
		pSelector->MouseEnter();
		pSelector->mDisplayText = mText;
	}
}

void SelectorItem::MouseLeave()
{
	if(mSelectable)
	{
		mColor = mNormalColor;
		pSelector->MouseLeave();
	}
}

void SelectorItem::MouseDown(int x, int y, int theBtnNum, int theClickCount)
{
}

void SelectorItem::MouseUp(int x, int y, int theBtnNum, int theClickCount)
{
	pSelector->pSourceObject->Delivered(mId);
	pSelector->MouseUp(x,y, theBtnNum, theClickCount);
}

