#include "TObjectFactory.h"
#include "GameApp.h"
#include "GameFrame.h"

#include "TObject.h"
#include "TObjectPizza.h"
#include "TObjectIngredient.h"
#include "TObjectOrder.h"

using namespace Sexy;

TObjectFactory::TObjectFactory(GameApp* theApp, GameFrame* theFrame)
{
	pApp = theApp;
	pFrame = theFrame;
}

TObjectFactory::~TObjectFactory()
{
}

TObject* TObjectFactory::getTObject(short name)
{
	TObject* to = NULL;

	switch(name)
	{
	case MARINARA:
	case CIPOLLA:
	case NAPOLETANA:
	case VIENNESE:
	case MARGHERITA:	to = new TObjectPizza(pApp, pFrame); 
						AssignPizzaIngredients(to, name); 
						break;
	
	case GARLIC:
	case ONION:
	case ANCHOVY:
	case SAUSAGE:
	case OREGANO:
	case BASIL:			to = new TObjectIngredient(pApp, pFrame); 
						AssignIngredientName(to, name);
						break;

	case ORDER:			to = new TObjectOrder(pApp, pFrame); 
						break;
	}

	if(to != NULL)
		AssignImages(to, name); 

	return to;
}

void TObjectFactory::AssignPizzaIngredients(TObject* theObj, short pizzaName)
{
	theObj->mType = pizzaName;
	
	switch(pizzaName)
	{
	case MARGHERITA:	theObj->mName = "Margherita";
						theObj->pImg = pApp->mPizzaMargheritaImg;
						theObj->mColor = Color(0,255,0);
						theObj->InitArray(1);
						theObj->mArray[0] = BASIL;
						break;

	case MARINARA:		theObj->mName = "Marinara";
						theObj->pImg = pApp->mPizzaMarinaraImg;
						theObj->mColor = Color(0,255,255);
						theObj->InitArray(3);
						theObj->mArray[0] = GARLIC;
						theObj->mArray[1] = OREGANO;
						theObj->mArray[2] = BASIL;
						break;

	case CIPOLLA:		theObj->mName = "Cipolla";
						theObj->pImg = pApp->mPizzaCipollaImg;
						theObj->mColor = Color(255,0,255);
						theObj->InitArray(1);
						theObj->mArray[0] = ONION;
						break;

	case NAPOLETANA:	theObj->mName = "Napoletana";
						theObj->pImg = pApp->mPizzaNapoletanaImg;
						theObj->mColor = Color(0,0,255);
						theObj->InitArray(2);
						theObj->mArray[0] = ANCHOVY;
						theObj->mArray[1] = OREGANO;
						break;

	case VIENNESE:		theObj->mName = "Viennese";
						theObj->pImg = pApp->mPizzaVienneseImg;
						theObj->mColor = Color(255,0,0);
						theObj->InitArray(2);
						theObj->mArray[0] = SAUSAGE;
						theObj->mArray[1] = OREGANO;
						break;
	}
}

void TObjectFactory::AssignIngredientName(TObject* theObj, short ingredientName)
{
	theObj->mType = ingredientName;
	
	switch(ingredientName)
	{
	case BASIL:			theObj->mName = "Basil";
						theObj->pImg = pApp->mBasilImg;
						break;

	case GARLIC:		theObj->mName = "Garlic";
						theObj->pImg = pApp->mGarlicImg;
						break;

	case OREGANO:		theObj->mName = "Oregano";
						theObj->pImg = pApp->mOreganoImg;
						break;

	case ONION:			theObj->mName = "Onions";
						theObj->pImg = pApp->mOnionImg;
						break;

	case ANCHOVY:		theObj->mName = "Anchovies";
						theObj->pImg = pApp->mAnchovyImg;
						break;

	case SAUSAGE:		theObj->mName = "Sausage";
						theObj->pImg = pApp->mSausageImg;
						break;
	}
}

void TObjectFactory::AssignImages(TObject* theObj, short theName)
{
	/*
	switch(theName)
	{

	}
	*/
}
