#include "GameApp.h"
#include "GameFrame.h"
#include "TObject.h"

#include "Tray.h"
#include "TrayWidget.h"

#include "../../../SexyAppFramework/Widget.h"

using namespace Sexy;

Tray::Tray(GameApp* theApp, GameFrame* theFrame, short theSize)
{
	pApp = theApp;
	pFrame = theFrame;

	mTObjArraySize = theSize;

	mTObjArray = new TObject*[mTObjArraySize];

	for(int i=0; i<mTObjArraySize; i++)
	{
		mTObjArray[i] = NULL;
	}

	mNumTObjects = 0;

	mCursorWidget = new TrayWidget(pApp, pFrame, this, mTObjArraySize);
	mUIWidget = new TrayWidget(pApp, pFrame, this, mTObjArraySize);
	mUIWidget->setVertical(false);
}

Tray::~Tray()
{
	for(int i=0; i<mTObjArraySize; i++)
	{
		if(mTObjArray[i] != NULL)
		{
			delete mTObjArray[i];
		}
	}

	delete mTObjArray;

	if(mCursorWidget != NULL)
	{
		delete mCursorWidget;
	}

	if(mUIWidget != NULL)
	{
		delete mUIWidget;
	}
}

Widget* Tray::getCursorWidget()
{
	return mCursorWidget;
}

Widget* Tray::getUIWidget()
{
	return mUIWidget;
}

short Tray::getNumElementInTrayOfId(short theId)
{
	short found = 0;

	for(int i=0; i<mNumTObjects; i++)
	{
		if(mTObjArray[i]->mType == theId)
		{
			found++;
		}
	}
	
	return found;
}

bool Tray::addItem(TObject* theObj)
{
	if(theObj != NULL)
	{
		if(mNumTObjects < mTObjArraySize)
		{
			mTObjArray[mNumTObjects] = theObj;
			mNumTObjects++;

			mCursorWidget->addItem(theObj);
			mUIWidget->addItem(theObj);

			//updateWidgets();

			return true;
		}
	}

	return false;
}

bool Tray::removeItem(TObject* theObj)
{
	if(mNumTObjects > 0)
	{
		short indexToRemove = -1;
		
		for(int i=0; i<mNumTObjects; i++)
		{
			if(mTObjArray[mNumTObjects-1-i] == theObj)
			{
				indexToRemove = mNumTObjects-1-i;
				break;
			}
		}

		if(indexToRemove != -1)
		{
			mCursorWidget->removeItem(mTObjArray[indexToRemove]);
			mUIWidget->removeItem(mTObjArray[indexToRemove]);
			
			for(int i=indexToRemove; i<mNumTObjects-1; i++)
			{
				mTObjArray[i] = mTObjArray[i+1];
			}
			
			mNumTObjects--;
			mTObjArray[mNumTObjects] = NULL;

			return true;
		}
	}
	
	return false;
}

bool Tray::hasSpace()
{
	if(mNumTObjects < mTObjArraySize)
		return true;

	return false;
}

bool Tray::isEmpty()
{
	if(mNumTObjects == 0)
		return true;

	return false;
}

TObject* Tray::getTopItem()
{
	if(mNumTObjects > 0)
	{
		return mTObjArray[mNumTObjects-1];
	}

	return NULL;
}

TObject* Tray::getTopItemById(short theId)
{
	if(mNumTObjects > 0)
	{
		if(getNumElementInTrayOfId(theId) > 0)
		{
			for(int j=0; j<mNumTObjects; j++)
			{
				if(mTObjArray[mNumTObjects-1-j]->mType == theId)
				{
					return (mTObjArray[mNumTObjects-1-j]);
					break;
				}
			}
		}
	}

	return NULL;
}

TObject* Tray::getItemByIndex(short theIndex)
{
	if(theIndex >= 0 && theIndex < mNumTObjects)
		return mTObjArray[theIndex];

	return NULL;
}

short Tray::getNumItemsInTray()
{
	return mNumTObjects;
}
